# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "ThirdPartyReachIntegrationPartnerEnum",
    },
)


class ThirdPartyReachIntegrationPartnerEnum(proto.Message):
    r"""Container for enum describing available third party
    integration partners for reach verification.

    """

    class ThirdPartyReachIntegrationPartner(proto.Enum):
        r"""Enum describing available third party integration partners
        for reach verification.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            NIELSEN (2):
                Nielsen.
            COMSCORE (3):
                Comscore.
            KANTAR_MILLWARD_BROWN (4):
                Kantar.
            VIDEO_RESEARCH (5):
                Video Research.
            GEMIUS (6):
                Gemius.
            MEDIA_SCOPE (7):
                MediaScope.
            AUDIENCE_PROJECT (8):
                AudienceProject
            VIDEO_AMP (9):
                VideoAmp
            ISPOT_TV (10):
                iSpot.tv
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NIELSEN = 2
        COMSCORE = 3
        KANTAR_MILLWARD_BROWN = 4
        VIDEO_RESEARCH = 5
        GEMIUS = 6
        MEDIA_SCOPE = 7
        AUDIENCE_PROJECT = 8
        VIDEO_AMP = 9
        ISPOT_TV = 10


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "ProductLinkInvitationStatusEnum",
    },
)


class ProductLinkInvitationStatusEnum(proto.Message):
    r"""Container for enum describing possible statuses of a product
    link invitation.

    """

    class ProductLinkInvitationStatus(proto.Enum):
        r"""Describes the possible statuses for an invitation between a
        Google Ads customer and another account.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ACCEPTED (2):
                The invitation is accepted.
            REQUESTED (3):
                An invitation has been sent to the other
                account. A user on the other account may now
                accept the invitation by setting the status to
                ACCEPTED.
            PENDING_APPROVAL (4):
                This invitation has been sent by a user on
                the other account. It may be accepted by a user
                on this account by setting the status to
                ACCEPTED.
            REVOKED (5):
                The invitation is revoked by the user who
                sent the invitation.
            REJECTED (6):
                The invitation has been rejected by the
                invitee.
            EXPIRED (7):
                The invitation has timed out before being
                accepted by the invitee.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCEPTED = 2
        REQUESTED = 3
        PENDING_APPROVAL = 4
        REVOKED = 5
        REJECTED = 6
        EXPIRED = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "RealTimeBiddingSetting",
    },
)


class RealTimeBiddingSetting(proto.Message):
    r"""Settings for Real-Time Bidding, a feature only available for
    campaigns targeting the Ad Exchange network.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        opt_in (bool):
            Whether the campaign is opted in to real-time
            bidding.

            This field is a member of `oneof`_ ``_opt_in``.
    """

    opt_in: bool = proto.Field(
        proto.BOOL,
        number=2,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AssetLinkPrimaryStatusEnum",
    },
)


class AssetLinkPrimaryStatusEnum(proto.Message):
    r"""Provides the primary status of an asset link.
    For example: a sitelink may be paused for a particular campaign.

    """

    class AssetLinkPrimaryStatus(proto.Enum):
        r"""Enum Provides insight into why an asset is not serving or not
        serving at full capacity for a particular link level. There
        could be one status with multiple reasons. For example, a
        sitelink might be paused by the user, but also limited in
        serving due to violation of an alcohol policy. In this case, the
        PrimaryStatus will be returned as PAUSED, since the asset's
        effective status is determined by its paused state.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ELIGIBLE (2):
                The asset is eligible to serve.
            PAUSED (3):
                The user-specified asset link status is
                paused.
            REMOVED (4):
                The user-specified asset link status is
                removed.
            PENDING (5):
                The asset may serve in the future.
            LIMITED (6):
                The asset is serving in a partial capacity.
            NOT_ELIGIBLE (7):
                The asset is not eligible to serve.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ELIGIBLE = 2
        PAUSED = 3
        REMOVED = 4
        PENDING = 5
        LIMITED = 6
        NOT_ELIGIBLE = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerFeedErrorEnum",
    },
)


class CustomerFeedErrorEnum(proto.Message):
    r"""Container for enum describing possible customer feed errors."""

    class CustomerFeedError(proto.Enum):
        r"""Enum describing possible customer feed errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE (2):
                An active feed already exists for this
                customer and place holder type.
            CANNOT_CREATE_FOR_REMOVED_FEED (3):
                The specified feed is removed.
            CANNOT_CREATE_ALREADY_EXISTING_CUSTOMER_FEED (4):
                The CustomerFeed already exists. Update
                should be used to modify the existing
                CustomerFeed.
            CANNOT_MODIFY_REMOVED_CUSTOMER_FEED (5):
                Cannot update removed customer feed.
            INVALID_PLACEHOLDER_TYPE (6):
                Invalid placeholder type.
            MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE (7):
                Feed mapping for this placeholder type does
                not exist.
            PLACEHOLDER_TYPE_NOT_ALLOWED_ON_CUSTOMER_FEED (8):
                Placeholder not allowed at the account level.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE = 2
        CANNOT_CREATE_FOR_REMOVED_FEED = 3
        CANNOT_CREATE_ALREADY_EXISTING_CUSTOMER_FEED = 4
        CANNOT_MODIFY_REMOVED_CUSTOMER_FEED = 5
        INVALID_PLACEHOLDER_TYPE = 6
        MISSING_FEEDMAPPING_FOR_PLACEHOLDER_TYPE = 7
        PLACEHOLDER_TYPE_NOT_ALLOWED_ON_CUSTOMER_FEED = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "InternalErrorEnum",
    },
)


class InternalErrorEnum(proto.Message):
    r"""Container for enum describing possible internal errors."""

    class InternalError(proto.Enum):
        r"""Enum describing possible internal errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            INTERNAL_ERROR (2):
                Google Ads API encountered unexpected
                internal error.
            ERROR_CODE_NOT_PUBLISHED (3):
                The intended error code doesn't exist in
                specified API version. It will be released in a
                future API version.
            TRANSIENT_ERROR (4):
                Google Ads API encountered an unexpected
                transient error. The user should retry their
                request in these cases.
            DEADLINE_EXCEEDED (5):
                The request took longer than a deadline.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        INTERNAL_ERROR = 2
        ERROR_CODE_NOT_PUBLISHED = 3
        TRANSIENT_ERROR = 4
        DEADLINE_EXCEEDED = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import offline_user_data
from google.ads.googleads.v22.enums.types import (
    offline_user_data_job_failure_reason,
)
from google.ads.googleads.v22.enums.types import (
    offline_user_data_job_match_rate_range,
)
from google.ads.googleads.v22.enums.types import offline_user_data_job_status
from google.ads.googleads.v22.enums.types import offline_user_data_job_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "OfflineUserDataJob",
        "OfflineUserDataJobMetadata",
    },
)


class OfflineUserDataJob(proto.Message):
    r"""A job containing offline user data of store visitors, or user
    list members that will be processed asynchronously. The uploaded
    data isn't readable and the processing results of the job can
    only be read using GoogleAdsService.Search/SearchStream.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the offline user data job.
            Offline user data job resource names have the form:

            ``customers/{customer_id}/offlineUserDataJobs/{offline_user_data_job_id}``
        id (int):
            Output only. ID of this offline user data
            job.

            This field is a member of `oneof`_ ``_id``.
        external_id (int):
            Immutable. User specified job ID.

            This field is a member of `oneof`_ ``_external_id``.
        type_ (google.ads.googleads.v22.enums.types.OfflineUserDataJobTypeEnum.OfflineUserDataJobType):
            Immutable. Type of the job.
        status (google.ads.googleads.v22.enums.types.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus):
            Output only. Status of the job.
        failure_reason (google.ads.googleads.v22.enums.types.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason):
            Output only. Reason for the processing
            failure, if status is FAILED.
        operation_metadata (google.ads.googleads.v22.resources.types.OfflineUserDataJobMetadata):
            Output only. Metadata of offline user data
            job depicting match rate range.
        customer_match_user_list_metadata (google.ads.googleads.v22.common.types.CustomerMatchUserListMetadata):
            Immutable. Metadata for data updates to a
            CRM-based user list.

            This field is a member of `oneof`_ ``metadata``.
        store_sales_metadata (google.ads.googleads.v22.common.types.StoreSalesMetadata):
            Immutable. Metadata for store sales data
            update.

            This field is a member of `oneof`_ ``metadata``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    external_id: int = proto.Field(
        proto.INT64,
        number=10,
        optional=True,
    )
    type_: (
        offline_user_data_job_type.OfflineUserDataJobTypeEnum.OfflineUserDataJobType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=offline_user_data_job_type.OfflineUserDataJobTypeEnum.OfflineUserDataJobType,
    )
    status: (
        offline_user_data_job_status.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=offline_user_data_job_status.OfflineUserDataJobStatusEnum.OfflineUserDataJobStatus,
    )
    failure_reason: (
        offline_user_data_job_failure_reason.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=offline_user_data_job_failure_reason.OfflineUserDataJobFailureReasonEnum.OfflineUserDataJobFailureReason,
    )
    operation_metadata: "OfflineUserDataJobMetadata" = proto.Field(
        proto.MESSAGE,
        number=11,
        message="OfflineUserDataJobMetadata",
    )
    customer_match_user_list_metadata: (
        offline_user_data.CustomerMatchUserListMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="metadata",
        message=offline_user_data.CustomerMatchUserListMetadata,
    )
    store_sales_metadata: offline_user_data.StoreSalesMetadata = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="metadata",
        message=offline_user_data.StoreSalesMetadata,
    )


class OfflineUserDataJobMetadata(proto.Message):
    r"""Metadata of offline user data job.

    Attributes:
        match_rate_range (google.ads.googleads.v22.enums.types.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange):
            Output only. Match rate of the Customer Match
            user list upload. Describes the estimated match
            rate when the status of the job is "RUNNING" and
            final match rate when the final match rate is
            available after the status of the job is
            "SUCCESS/FAILED".
    """

    match_rate_range: (
        offline_user_data_job_match_rate_range.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=offline_user_data_job_match_rate_range.OfflineUserDataJobMatchRateRangeEnum.OfflineUserDataJobMatchRateRange,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Updates the access role of a user, given the email address.

This code example should be run as a user who is an Administrator on the Google
Ads account with the specified customer ID.

See https://support.google.com/google-ads/answer/9978556 to learn more about
account access levels.
"""

import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsRequest,
    SearchPagedResponse,
)
from google.ads.googleads.v22.resources.types.customer_user_access import (
    CustomerUserAccess,
)
from google.ads.googleads.v22.services.services.customer_user_access_service.client import (
    CustomerUserAccessServiceClient,
)
from google.ads.googleads.v22.services.types.customer_user_access_service import (
    CustomerUserAccessOperation,
    MutateCustomerUserAccessResponse,
)

from google.api_core import protobuf_helpers
from google.protobuf.field_mask_pb2 import FieldMask

from typing import Optional

_ACCESS_ROLES = ["ADMIN", "STANDARD", "READ_ONLY", "EMAIL_ONLY"]


def main(
    client: GoogleAdsClient,
    customer_id: str,
    email_address: str,
    access_role: str,
) -> None:
    """Runs the example.

    Args:
      client: The Google Ads client.
      customer_id: The customer ID.
      email_address: The email address of the user whose access role should
          be updated
      access_role: The updated access role.
    """

    user_id: Optional[int] = get_user_access(client, customer_id, email_address)

    if user_id:
        modify_user_access(client, customer_id, user_id, access_role)


def get_user_access(
    client: GoogleAdsClient, customer_id: str, email_address: str
) -> Optional[int]:
    """Gets the customer user access given an email address.

    Args:
      client: The Google Ads client.
      customer_id: The customer ID.
      email_address: The email address of the user whose access role should
          be updated.

    Returns:
      The user ID integer if a customer is found, otherwise None.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # Creates a query that retrieves all customer user accesses.
    # Use the LIKE query for filtering to ignore the text case for email
    # address when searching for a match.
    query = f"""
        SELECT
          customer_user_access.user_id,
          customer_user_access.email_address,
          customer_user_access.access_role,
          customer_user_access.access_creation_date_time
        FROM customer_user_access
        WHERE customer_user_access.email_address LIKE '{email_address}'"""

    search_request: SearchGoogleAdsRequest = client.get_type(
        "SearchGoogleAdsRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query

    response: SearchPagedResponse = googleads_service.search(
        request=search_request
    )

    try:
        # Assuming response is an iterable of GoogleAdsRow, and each row has customer_user_access
        # For safety, check if response has items, though next(iter()) handles empty.
        user_access: CustomerUserAccess = next(
            iter(response)
        ).customer_user_access
        print(
            "Customer user access with "
            f"User ID = '{user_access.user_id}', "
            f"Access Role = '{user_access.access_role}', and "
            f"Creation Time = {user_access.access_creation_date_time} "
            f"was found in Customer ID: {customer_id}."
        )
        return user_access.user_id
    except StopIteration:
        # If a StopIteration exception is raised it indicates that the response
        # was empty, no results were found, and this method should return None.
        print("No customer user access with requested email was found.")
        return None


def modify_user_access(
    client: GoogleAdsClient, customer_id: str, user_id: int, access_role: str
) -> None:
    """Modifies the user access role to a specified value.

    Args:
      client: The Google Ads client.
      customer_id: The customer ID.
      user_id: ID of the user whose access role is being modified.
      access_role: The updated access role.
    """
    customer_user_access_service: CustomerUserAccessServiceClient = (
        client.get_service("CustomerUserAccessService")
    )
    customer_user_access_op: CustomerUserAccessOperation = client.get_type(
        "CustomerUserAccessOperation"
    )
    # access_role_enum is client.enums.AccessRoleEnum, which is an EnumTypeWrapper.
    # We use getattr to get the specific enum member by its string name.
    # The type of access_role_enum itself is complex if fully specified, using it directly is fine.
    access_role_enum = client.enums.AccessRoleEnum
    customer_user_access: CustomerUserAccess = customer_user_access_op.update
    customer_user_access.resource_name = (
        customer_user_access_service.customer_user_access_path(
            customer_id, user_id
        )
    )
    # customer_user_access.access_role expects an AccessRoleEnum value (int)
    customer_user_access.access_role = getattr(
        access_role_enum, access_role
    ).value  # Use .value for int
    update_mask: FieldMask = protobuf_helpers.field_mask(
        None, customer_user_access._pb
    )
    client.copy_from(
        customer_user_access_op.update_mask,
        update_mask,
    )

    response: MutateCustomerUserAccessResponse = (
        customer_user_access_service.mutate_customer_user_access(
            customer_id=customer_id, operation=customer_user_access_op
        )
    )

    print(
        "Successfully modified customer user access with resource name: "
        f"{response.result.resource_name}."
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="This code example updates the access role of a user, "
        "given the email address."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-e",
        "--email_address",
        type=str,
        required=True,
        help="The email address of the user whose access role should be "
        "updated.",
    )
    parser.add_argument(
        "-a",
        "--access_role",
        type=str,
        required=True,
        help="The access role that the given email address should be set to.",
        choices=_ACCESS_ROLES,
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")
    try:
        main(
            googleads_client,
            args.customer_id,
            args.email_address,
            args.access_role,
        )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "MediaBundleErrorEnum",
    },
)


class MediaBundleErrorEnum(proto.Message):
    r"""Container for enum describing possible media bundle errors."""

    class MediaBundleError(proto.Enum):
        r"""Enum describing possible media bundle errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            BAD_REQUEST (3):
                There was a problem with the request.
            DOUBLECLICK_BUNDLE_NOT_ALLOWED (4):
                HTML5 ads using DoubleClick Studio created
                ZIP files are not supported.
            EXTERNAL_URL_NOT_ALLOWED (5):
                Cannot reference URL external to the media
                bundle.
            FILE_TOO_LARGE (6):
                Media bundle file is too large.
            GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED (7):
                ZIP file from Google Web Designer is not
                published.
            INVALID_INPUT (8):
                Input was invalid.
            INVALID_MEDIA_BUNDLE (9):
                There was a problem with the media bundle.
            INVALID_MEDIA_BUNDLE_ENTRY (10):
                There was a problem with one or more of the
                media bundle entries.
            INVALID_MIME_TYPE (11):
                The media bundle contains a file with an
                unknown mime type
            INVALID_PATH (12):
                The media bundle contain an invalid asset
                path.
            INVALID_URL_REFERENCE (13):
                HTML5 ad is trying to reference an asset not
                in .ZIP file
            MEDIA_DATA_TOO_LARGE (14):
                Media data is too large.
            MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY (15):
                The media bundle contains no primary entry.
            SERVER_ERROR (16):
                There was an error on the server.
            STORAGE_ERROR (17):
                The image could not be stored.
            SWIFFY_BUNDLE_NOT_ALLOWED (18):
                Media bundle created with the Swiffy tool is
                not allowed.
            TOO_MANY_FILES (19):
                The media bundle contains too many files.
            UNEXPECTED_SIZE (20):
                The media bundle is not of legal dimensions.
            UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT (21):
                Google Web Designer not created for "Google
                Ads" environment.
            UNSUPPORTED_HTML5_FEATURE (22):
                Unsupported HTML5 feature in HTML5 asset.
            URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT (23):
                URL in HTML5 entry is not ssl compliant.
            CUSTOM_EXIT_NOT_ALLOWED (24):
                Custom exits not allowed in HTML5 entry.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BAD_REQUEST = 3
        DOUBLECLICK_BUNDLE_NOT_ALLOWED = 4
        EXTERNAL_URL_NOT_ALLOWED = 5
        FILE_TOO_LARGE = 6
        GOOGLE_WEB_DESIGNER_ZIP_FILE_NOT_PUBLISHED = 7
        INVALID_INPUT = 8
        INVALID_MEDIA_BUNDLE = 9
        INVALID_MEDIA_BUNDLE_ENTRY = 10
        INVALID_MIME_TYPE = 11
        INVALID_PATH = 12
        INVALID_URL_REFERENCE = 13
        MEDIA_DATA_TOO_LARGE = 14
        MISSING_PRIMARY_MEDIA_BUNDLE_ENTRY = 15
        SERVER_ERROR = 16
        STORAGE_ERROR = 17
        SWIFFY_BUNDLE_NOT_ALLOWED = 18
        TOO_MANY_FILES = 19
        UNEXPECTED_SIZE = 20
        UNSUPPORTED_GOOGLE_WEB_DESIGNER_ENVIRONMENT = 21
        UNSUPPORTED_HTML5_FEATURE = 22
        URL_IN_MEDIA_BUNDLE_NOT_SSL_COMPLIANT = 23
        CUSTOM_EXIT_NOT_ALLOWED = 24


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .access_invitation_status import (
    AccessInvitationStatusEnum,
)
from .access_reason import (
    AccessReasonEnum,
)
from .access_role import (
    AccessRoleEnum,
)
from .account_budget_proposal_status import (
    AccountBudgetProposalStatusEnum,
)
from .account_budget_proposal_type import (
    AccountBudgetProposalTypeEnum,
)
from .account_budget_status import (
    AccountBudgetStatusEnum,
)
from .account_link_status import (
    AccountLinkStatusEnum,
)
from .ad_destination_type import (
    AdDestinationTypeEnum,
)
from .ad_format_type import (
    AdFormatTypeEnum,
)
from .ad_group_ad_primary_status import (
    AdGroupAdPrimaryStatusEnum,
)
from .ad_group_ad_primary_status_reason import (
    AdGroupAdPrimaryStatusReasonEnum,
)
from .ad_group_ad_rotation_mode import (
    AdGroupAdRotationModeEnum,
)
from .ad_group_ad_status import (
    AdGroupAdStatusEnum,
)
from .ad_group_criterion_approval_status import (
    AdGroupCriterionApprovalStatusEnum,
)
from .ad_group_criterion_primary_status import (
    AdGroupCriterionPrimaryStatusEnum,
)
from .ad_group_criterion_primary_status_reason import (
    AdGroupCriterionPrimaryStatusReasonEnum,
)
from .ad_group_criterion_status import (
    AdGroupCriterionStatusEnum,
)
from .ad_group_primary_status import (
    AdGroupPrimaryStatusEnum,
)
from .ad_group_primary_status_reason import (
    AdGroupPrimaryStatusReasonEnum,
)
from .ad_group_status import (
    AdGroupStatusEnum,
)
from .ad_group_type import (
    AdGroupTypeEnum,
)
from .ad_network_type import (
    AdNetworkTypeEnum,
)
from .ad_serving_optimization_status import (
    AdServingOptimizationStatusEnum,
)
from .ad_strength import (
    AdStrengthEnum,
)
from .ad_strength_action_item_type import (
    AdStrengthActionItemTypeEnum,
)
from .ad_type import (
    AdTypeEnum,
)
from .advertising_channel_sub_type import (
    AdvertisingChannelSubTypeEnum,
)
from .advertising_channel_type import (
    AdvertisingChannelTypeEnum,
)
from .age_range_type import (
    AgeRangeTypeEnum,
)
from .android_privacy_interaction_type import (
    AndroidPrivacyInteractionTypeEnum,
)
from .android_privacy_network_type import (
    AndroidPrivacyNetworkTypeEnum,
)
from .app_bidding_goal import (
    AppBiddingGoalEnum,
)
from .app_campaign_app_store import (
    AppCampaignAppStoreEnum,
)
from .app_campaign_bidding_strategy_goal_type import (
    AppCampaignBiddingStrategyGoalTypeEnum,
)
from .app_payment_model_type import (
    AppPaymentModelTypeEnum,
)
from .app_url_operating_system_type import (
    AppUrlOperatingSystemTypeEnum,
)
from .application_instance import (
    ApplicationInstanceEnum,
)
from .asset_automation_status import (
    AssetAutomationStatusEnum,
)
from .asset_automation_type import (
    AssetAutomationTypeEnum,
)
from .asset_coverage_video_aspect_ratio_requirement import (
    AssetCoverageVideoAspectRatioRequirementEnum,
)
from .asset_field_type import (
    AssetFieldTypeEnum,
)
from .asset_group_primary_status import (
    AssetGroupPrimaryStatusEnum,
)
from .asset_group_primary_status_reason import (
    AssetGroupPrimaryStatusReasonEnum,
)
from .asset_group_signal_approval_status import (
    AssetGroupSignalApprovalStatusEnum,
)
from .asset_group_status import (
    AssetGroupStatusEnum,
)
from .asset_link_primary_status import (
    AssetLinkPrimaryStatusEnum,
)
from .asset_link_primary_status_reason import (
    AssetLinkPrimaryStatusReasonEnum,
)
from .asset_link_status import (
    AssetLinkStatusEnum,
)
from .asset_offline_evaluation_error_reasons import (
    AssetOfflineEvaluationErrorReasonsEnum,
)
from .asset_performance_label import (
    AssetPerformanceLabelEnum,
)
from .asset_set_asset_status import (
    AssetSetAssetStatusEnum,
)
from .asset_set_link_status import (
    AssetSetLinkStatusEnum,
)
from .asset_set_status import (
    AssetSetStatusEnum,
)
from .asset_set_type import (
    AssetSetTypeEnum,
)
from .asset_source import (
    AssetSourceEnum,
)
from .asset_type import (
    AssetTypeEnum,
)
from .async_action_status import (
    AsyncActionStatusEnum,
)
from .attribution_model import (
    AttributionModelEnum,
)
from .audience_insights_dimension import (
    AudienceInsightsDimensionEnum,
)
from .audience_insights_marketing_objective import (
    AudienceInsightsMarketingObjectiveEnum,
)
from .audience_scope import (
    AudienceScopeEnum,
)
from .audience_status import (
    AudienceStatusEnum,
)
from .batch_job_status import (
    BatchJobStatusEnum,
)
from .bid_modifier_source import (
    BidModifierSourceEnum,
)
from .bidding_source import (
    BiddingSourceEnum,
)
from .bidding_strategy_status import (
    BiddingStrategyStatusEnum,
)
from .bidding_strategy_system_status import (
    BiddingStrategySystemStatusEnum,
)
from .bidding_strategy_type import (
    BiddingStrategyTypeEnum,
)
from .billing_setup_status import (
    BillingSetupStatusEnum,
)
from .brand_request_rejection_reason import (
    BrandRequestRejectionReasonEnum,
)
from .brand_safety_suitability import (
    BrandSafetySuitabilityEnum,
)
from .brand_state import (
    BrandStateEnum,
)
from .budget_campaign_association_status import (
    BudgetCampaignAssociationStatusEnum,
)
from .budget_delivery_method import (
    BudgetDeliveryMethodEnum,
)
from .budget_period import (
    BudgetPeriodEnum,
)
from .budget_status import (
    BudgetStatusEnum,
)
from .budget_type import (
    BudgetTypeEnum,
)
from .business_message_call_to_action_type import (
    BusinessMessageCallToActionTypeEnum,
)
from .business_message_provider import (
    BusinessMessageProviderEnum,
)
from .call_conversion_reporting_state import (
    CallConversionReportingStateEnum,
)
from .call_to_action_type import (
    CallToActionTypeEnum,
)
from .call_tracking_display_location import (
    CallTrackingDisplayLocationEnum,
)
from .call_type import (
    CallTypeEnum,
)
from .campaign_criterion_status import (
    CampaignCriterionStatusEnum,
)
from .campaign_draft_status import (
    CampaignDraftStatusEnum,
)
from .campaign_experiment_type import (
    CampaignExperimentTypeEnum,
)
from .campaign_group_status import (
    CampaignGroupStatusEnum,
)
from .campaign_keyword_match_type import (
    CampaignKeywordMatchTypeEnum,
)
from .campaign_primary_status import (
    CampaignPrimaryStatusEnum,
)
from .campaign_primary_status_reason import (
    CampaignPrimaryStatusReasonEnum,
)
from .campaign_serving_status import (
    CampaignServingStatusEnum,
)
from .campaign_shared_set_status import (
    CampaignSharedSetStatusEnum,
)
from .campaign_status import (
    CampaignStatusEnum,
)
from .chain_relationship_type import (
    ChainRelationshipTypeEnum,
)
from .change_client_type import (
    ChangeClientTypeEnum,
)
from .change_event_resource_type import (
    ChangeEventResourceTypeEnum,
)
from .change_status_operation import (
    ChangeStatusOperationEnum,
)
from .change_status_resource_type import (
    ChangeStatusResourceTypeEnum,
)
from .click_type import (
    ClickTypeEnum,
)
from .combined_audience_status import (
    CombinedAudienceStatusEnum,
)
from .consent_status import (
    ConsentStatusEnum,
)
from .content_label_type import (
    ContentLabelTypeEnum,
)
from .conversion_action_category import (
    ConversionActionCategoryEnum,
)
from .conversion_action_counting_type import (
    ConversionActionCountingTypeEnum,
)
from .conversion_action_status import (
    ConversionActionStatusEnum,
)
from .conversion_action_type import (
    ConversionActionTypeEnum,
)
from .conversion_adjustment_type import (
    ConversionAdjustmentTypeEnum,
)
from .conversion_attribution_event_type import (
    ConversionAttributionEventTypeEnum,
)
from .conversion_custom_variable_status import (
    ConversionCustomVariableStatusEnum,
)
from .conversion_customer_type import (
    ConversionCustomerTypeEnum,
)
from .conversion_environment_enum import (
    ConversionEnvironmentEnum,
)
from .conversion_lag_bucket import (
    ConversionLagBucketEnum,
)
from .conversion_or_adjustment_lag_bucket import (
    ConversionOrAdjustmentLagBucketEnum,
)
from .conversion_origin import (
    ConversionOriginEnum,
)
from .conversion_tracking_status_enum import (
    ConversionTrackingStatusEnum,
)
from .conversion_value_rule_primary_dimension import (
    ConversionValueRulePrimaryDimensionEnum,
)
from .conversion_value_rule_set_status import (
    ConversionValueRuleSetStatusEnum,
)
from .conversion_value_rule_status import (
    ConversionValueRuleStatusEnum,
)
from .converting_user_prior_engagement_type_and_ltv_bucket import (
    ConvertingUserPriorEngagementTypeAndLtvBucketEnum,
)
from .criterion_category_channel_availability_mode import (
    CriterionCategoryChannelAvailabilityModeEnum,
)
from .criterion_category_locale_availability_mode import (
    CriterionCategoryLocaleAvailabilityModeEnum,
)
from .criterion_system_serving_status import (
    CriterionSystemServingStatusEnum,
)
from .criterion_type import (
    CriterionTypeEnum,
)
from .custom_audience_member_type import (
    CustomAudienceMemberTypeEnum,
)
from .custom_audience_status import (
    CustomAudienceStatusEnum,
)
from .custom_audience_type import (
    CustomAudienceTypeEnum,
)
from .custom_conversion_goal_status import (
    CustomConversionGoalStatusEnum,
)
from .custom_interest_member_type import (
    CustomInterestMemberTypeEnum,
)
from .custom_interest_status import (
    CustomInterestStatusEnum,
)
from .custom_interest_type import (
    CustomInterestTypeEnum,
)
from .customer_acquisition_optimization_mode import (
    CustomerAcquisitionOptimizationModeEnum,
)
from .customer_match_upload_key_type import (
    CustomerMatchUploadKeyTypeEnum,
)
from .customer_pay_per_conversion_eligibility_failure_reason import (
    CustomerPayPerConversionEligibilityFailureReasonEnum,
)
from .customer_status import (
    CustomerStatusEnum,
)
from .customizer_attribute_status import (
    CustomizerAttributeStatusEnum,
)
from .customizer_attribute_type import (
    CustomizerAttributeTypeEnum,
)
from .customizer_value_status import (
    CustomizerValueStatusEnum,
)
from .data_driven_model_status import (
    DataDrivenModelStatusEnum,
)
from .data_link_status import (
    DataLinkStatusEnum,
)
from .data_link_type import (
    DataLinkTypeEnum,
)
from .day_of_week import (
    DayOfWeekEnum,
)
from .demand_gen_channel_config import (
    DemandGenChannelConfigEnum,
)
from .demand_gen_channel_strategy import (
    DemandGenChannelStrategyEnum,
)
from .device import (
    DeviceEnum,
)
from .display_ad_format_setting import (
    DisplayAdFormatSettingEnum,
)
from .display_upload_product_type import (
    DisplayUploadProductTypeEnum,
)
from .distance_bucket import (
    DistanceBucketEnum,
)
from .eu_political_advertising_status import (
    EuPoliticalAdvertisingStatusEnum,
)
from .experiment_metric import (
    ExperimentMetricEnum,
)
from .experiment_metric_direction import (
    ExperimentMetricDirectionEnum,
)
from .experiment_status import (
    ExperimentStatusEnum,
)
from .experiment_type import (
    ExperimentTypeEnum,
)
from .external_conversion_source import (
    ExternalConversionSourceEnum,
)
from .fixed_cpm_goal import (
    FixedCpmGoalEnum,
)
from .fixed_cpm_target_frequency_time_unit import (
    FixedCpmTargetFrequencyTimeUnitEnum,
)
from .frequency_cap_event_type import (
    FrequencyCapEventTypeEnum,
)
from .frequency_cap_level import (
    FrequencyCapLevelEnum,
)
from .frequency_cap_time_unit import (
    FrequencyCapTimeUnitEnum,
)
from .gender_type import (
    GenderTypeEnum,
)
from .geo_target_constant_status import (
    GeoTargetConstantStatusEnum,
)
from .geo_targeting_type import (
    GeoTargetingTypeEnum,
)
from .goal_config_level import (
    GoalConfigLevelEnum,
)
from .google_ads_field_category import (
    GoogleAdsFieldCategoryEnum,
)
from .google_ads_field_data_type import (
    GoogleAdsFieldDataTypeEnum,
)
from .google_voice_call_status import (
    GoogleVoiceCallStatusEnum,
)
from .hotel_asset_suggestion_status import (
    HotelAssetSuggestionStatusEnum,
)
from .hotel_date_selection_type import (
    HotelDateSelectionTypeEnum,
)
from .hotel_price_bucket import (
    HotelPriceBucketEnum,
)
from .hotel_rate_type import (
    HotelRateTypeEnum,
)
from .hotel_reconciliation_status import (
    HotelReconciliationStatusEnum,
)
from .identity_verification_program import (
    IdentityVerificationProgramEnum,
)
from .identity_verification_program_status import (
    IdentityVerificationProgramStatusEnum,
)
from .income_range_type import (
    IncomeRangeTypeEnum,
)
from .insights_knowledge_graph_entity_capabilities import (
    InsightsKnowledgeGraphEntityCapabilitiesEnum,
)
from .insights_trend import (
    InsightsTrendEnum,
)
from .interaction_event_type import (
    InteractionEventTypeEnum,
)
from .interaction_type import (
    InteractionTypeEnum,
)
from .invoice_type import (
    InvoiceTypeEnum,
)
from .keyword_match_type import (
    KeywordMatchTypeEnum,
)
from .keyword_plan_aggregate_metric_type import (
    KeywordPlanAggregateMetricTypeEnum,
)
from .keyword_plan_competition_level import (
    KeywordPlanCompetitionLevelEnum,
)
from .keyword_plan_concept_group_type import (
    KeywordPlanConceptGroupTypeEnum,
)
from .keyword_plan_forecast_interval import (
    KeywordPlanForecastIntervalEnum,
)
from .keyword_plan_keyword_annotation import (
    KeywordPlanKeywordAnnotationEnum,
)
from .keyword_plan_network import (
    KeywordPlanNetworkEnum,
)
from .label_status import (
    LabelStatusEnum,
)
from .lead_form_call_to_action_type import (
    LeadFormCallToActionTypeEnum,
)
from .lead_form_desired_intent import (
    LeadFormDesiredIntentEnum,
)
from .lead_form_field_user_input_type import (
    LeadFormFieldUserInputTypeEnum,
)
from .lead_form_post_submit_call_to_action_type import (
    LeadFormPostSubmitCallToActionTypeEnum,
)
from .legacy_app_install_ad_app_store import (
    LegacyAppInstallAdAppStoreEnum,
)
from .linked_account_type import (
    LinkedAccountTypeEnum,
)
from .linked_product_type import (
    LinkedProductTypeEnum,
)
from .listing_group_filter_custom_attribute_index import (
    ListingGroupFilterCustomAttributeIndexEnum,
)
from .listing_group_filter_listing_source import (
    ListingGroupFilterListingSourceEnum,
)
from .listing_group_filter_product_category_level import (
    ListingGroupFilterProductCategoryLevelEnum,
)
from .listing_group_filter_product_channel import (
    ListingGroupFilterProductChannelEnum,
)
from .listing_group_filter_product_condition import (
    ListingGroupFilterProductConditionEnum,
)
from .listing_group_filter_product_type_level import (
    ListingGroupFilterProductTypeLevelEnum,
)
from .listing_group_filter_type_enum import (
    ListingGroupFilterTypeEnum,
)
from .listing_group_type import (
    ListingGroupTypeEnum,
)
from .listing_type import (
    ListingTypeEnum,
)
from .local_services_business_registration_check_rejection_reason import (
    LocalServicesBusinessRegistrationCheckRejectionReasonEnum,
)
from .local_services_business_registration_type import (
    LocalServicesBusinessRegistrationTypeEnum,
)
from .local_services_conversation_type import (
    LocalServicesLeadConversationTypeEnum,
)
from .local_services_employee_status import (
    LocalServicesEmployeeStatusEnum,
)
from .local_services_employee_type import (
    LocalServicesEmployeeTypeEnum,
)
from .local_services_insurance_rejection_reason import (
    LocalServicesInsuranceRejectionReasonEnum,
)
from .local_services_lead_credit_issuance_decision import (
    LocalServicesLeadCreditIssuanceDecisionEnum,
)
from .local_services_lead_credit_state import (
    LocalServicesCreditStateEnum,
)
from .local_services_lead_status import (
    LocalServicesLeadStatusEnum,
)
from .local_services_lead_survey_answer import (
    LocalServicesLeadSurveyAnswerEnum,
)
from .local_services_lead_survey_dissatisfied_reason import (
    LocalServicesLeadSurveyDissatisfiedReasonEnum,
)
from .local_services_lead_survey_satisfied_reason import (
    LocalServicesLeadSurveySatisfiedReasonEnum,
)
from .local_services_lead_type import (
    LocalServicesLeadTypeEnum,
)
from .local_services_license_rejection_reason import (
    LocalServicesLicenseRejectionReasonEnum,
)
from .local_services_participant_type import (
    LocalServicesParticipantTypeEnum,
)
from .local_services_verification_artifact_status import (
    LocalServicesVerificationArtifactStatusEnum,
)
from .local_services_verification_artifact_type import (
    LocalServicesVerificationArtifactTypeEnum,
)
from .local_services_verification_status import (
    LocalServicesVerificationStatusEnum,
)
from .location_group_radius_units import (
    LocationGroupRadiusUnitsEnum,
)
from .location_ownership_type import (
    LocationOwnershipTypeEnum,
)
from .location_source_type import (
    LocationSourceTypeEnum,
)
from .location_string_filter_type import (
    LocationStringFilterTypeEnum,
)
from .lookalike_expansion_level import (
    LookalikeExpansionLevelEnum,
)
from .manager_link_status import (
    ManagerLinkStatusEnum,
)
from .media_type import (
    MediaTypeEnum,
)
from .mime_type import (
    MimeTypeEnum,
)
from .minute_of_hour import (
    MinuteOfHourEnum,
)
from .mobile_app_vendor import (
    MobileAppVendorEnum,
)
from .mobile_device_type import (
    MobileDeviceTypeEnum,
)
from .month_of_year import (
    MonthOfYearEnum,
)
from .negative_geo_target_type import (
    NegativeGeoTargetTypeEnum,
)
from .non_skippable_max_duration import (
    NonSkippableMaxDurationEnum,
)
from .non_skippable_min_duration import (
    NonSkippableMinDurationEnum,
)
from .offline_conversion_diagnostic_status_enum import (
    OfflineConversionDiagnosticStatusEnum,
)
from .offline_event_upload_client_enum import (
    OfflineEventUploadClientEnum,
)
from .offline_user_data_job_failure_reason import (
    OfflineUserDataJobFailureReasonEnum,
)
from .offline_user_data_job_match_rate_range import (
    OfflineUserDataJobMatchRateRangeEnum,
)
from .offline_user_data_job_status import (
    OfflineUserDataJobStatusEnum,
)
from .offline_user_data_job_type import (
    OfflineUserDataJobTypeEnum,
)
from .operating_system_version_operator_type import (
    OperatingSystemVersionOperatorTypeEnum,
)
from .optimization_goal_type import (
    OptimizationGoalTypeEnum,
)
from .parental_status_type import (
    ParentalStatusTypeEnum,
)
from .payment_mode import (
    PaymentModeEnum,
)
from .performance_max_upgrade_status import (
    PerformanceMaxUpgradeStatusEnum,
)
from .placement_type import (
    PlacementTypeEnum,
)
from .policy_approval_status import (
    PolicyApprovalStatusEnum,
)
from .policy_review_status import (
    PolicyReviewStatusEnum,
)
from .policy_topic_entry_type import (
    PolicyTopicEntryTypeEnum,
)
from .policy_topic_evidence_destination_mismatch_url_type import (
    PolicyTopicEvidenceDestinationMismatchUrlTypeEnum,
)
from .policy_topic_evidence_destination_not_working_device import (
    PolicyTopicEvidenceDestinationNotWorkingDeviceEnum,
)
from .policy_topic_evidence_destination_not_working_dns_error_type import (
    PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum,
)
from .positive_geo_target_type import (
    PositiveGeoTargetTypeEnum,
)
from .price_extension_price_qualifier import (
    PriceExtensionPriceQualifierEnum,
)
from .price_extension_price_unit import (
    PriceExtensionPriceUnitEnum,
)
from .price_extension_type import (
    PriceExtensionTypeEnum,
)
from .product_availability import (
    ProductAvailabilityEnum,
)
from .product_category_level import (
    ProductCategoryLevelEnum,
)
from .product_category_state import (
    ProductCategoryStateEnum,
)
from .product_channel import (
    ProductChannelEnum,
)
from .product_channel_exclusivity import (
    ProductChannelExclusivityEnum,
)
from .product_condition import (
    ProductConditionEnum,
)
from .product_custom_attribute_index import (
    ProductCustomAttributeIndexEnum,
)
from .product_issue_severity import (
    ProductIssueSeverityEnum,
)
from .product_link_invitation_status import (
    ProductLinkInvitationStatusEnum,
)
from .product_status import (
    ProductStatusEnum,
)
from .product_type_level import (
    ProductTypeLevelEnum,
)
from .promotion_extension_discount_modifier import (
    PromotionExtensionDiscountModifierEnum,
)
from .promotion_extension_occasion import (
    PromotionExtensionOccasionEnum,
)
from .proximity_radius_units import (
    ProximityRadiusUnitsEnum,
)
from .quality_score_bucket import (
    QualityScoreBucketEnum,
)
from .reach_plan_age_range import (
    ReachPlanAgeRangeEnum,
)
from .reach_plan_conversion_rate_model import (
    ReachPlanConversionRateModelEnum,
)
from .reach_plan_network import (
    ReachPlanNetworkEnum,
)
from .reach_plan_plannable_user_list_status import (
    ReachPlanPlannableUserListStatusEnum,
)
from .reach_plan_surface import (
    ReachPlanSurfaceEnum,
)
from .recommendation_subscription_status import (
    RecommendationSubscriptionStatusEnum,
)
from .recommendation_type import (
    RecommendationTypeEnum,
)
from .resource_change_operation import (
    ResourceChangeOperationEnum,
)
from .resource_limit_type import (
    ResourceLimitTypeEnum,
)
from .response_content_type import (
    ResponseContentTypeEnum,
)
from .search_engine_results_page_type import (
    SearchEngineResultsPageTypeEnum,
)
from .search_term_match_type import (
    SearchTermMatchTypeEnum,
)
from .search_term_targeting_status import (
    SearchTermTargetingStatusEnum,
)
from .seasonality_event_scope import (
    SeasonalityEventScopeEnum,
)
from .seasonality_event_status import (
    SeasonalityEventStatusEnum,
)
from .served_asset_field_type import (
    ServedAssetFieldTypeEnum,
)
from .shared_set_status import (
    SharedSetStatusEnum,
)
from .shared_set_type import (
    SharedSetTypeEnum,
)
from .shopping_add_products_to_campaign_recommendation_enum import (
    ShoppingAddProductsToCampaignRecommendationEnum,
)
from .simulation_modification_method import (
    SimulationModificationMethodEnum,
)
from .simulation_type import (
    SimulationTypeEnum,
)
from .sk_ad_network_ad_event_type import (
    SkAdNetworkAdEventTypeEnum,
)
from .sk_ad_network_attribution_credit import (
    SkAdNetworkAttributionCreditEnum,
)
from .sk_ad_network_coarse_conversion_value import (
    SkAdNetworkCoarseConversionValueEnum,
)
from .sk_ad_network_source_type import (
    SkAdNetworkSourceTypeEnum,
)
from .sk_ad_network_user_type import (
    SkAdNetworkUserTypeEnum,
)
from .slot import (
    SlotEnum,
)
from .smart_campaign_not_eligible_reason import (
    SmartCampaignNotEligibleReasonEnum,
)
from .smart_campaign_status import (
    SmartCampaignStatusEnum,
)
from .spending_limit_type import (
    SpendingLimitTypeEnum,
)
from .summary_row_setting import (
    SummaryRowSettingEnum,
)
from .system_managed_entity_source import (
    SystemManagedResourceSourceEnum,
)
from .target_cpa_opt_in_recommendation_goal import (
    TargetCpaOptInRecommendationGoalEnum,
)
from .target_frequency_time_unit import (
    TargetFrequencyTimeUnitEnum,
)
from .target_impression_share_location import (
    TargetImpressionShareLocationEnum,
)
from .targeting_dimension import (
    TargetingDimensionEnum,
)
from .time_type import (
    TimeTypeEnum,
)
from .tracking_code_page_format import (
    TrackingCodePageFormatEnum,
)
from .tracking_code_type import (
    TrackingCodeTypeEnum,
)
from .user_identifier_source import (
    UserIdentifierSourceEnum,
)
from .user_interest_taxonomy_type import (
    UserInterestTaxonomyTypeEnum,
)
from .user_list_access_status import (
    UserListAccessStatusEnum,
)
from .user_list_closing_reason import (
    UserListClosingReasonEnum,
)
from .user_list_crm_data_source_type import (
    UserListCrmDataSourceTypeEnum,
)
from .user_list_customer_type_category import (
    UserListCustomerTypeCategoryEnum,
)
from .user_list_date_rule_item_operator import (
    UserListDateRuleItemOperatorEnum,
)
from .user_list_flexible_rule_operator import (
    UserListFlexibleRuleOperatorEnum,
)
from .user_list_logical_rule_operator import (
    UserListLogicalRuleOperatorEnum,
)
from .user_list_membership_status import (
    UserListMembershipStatusEnum,
)
from .user_list_number_rule_item_operator import (
    UserListNumberRuleItemOperatorEnum,
)
from .user_list_prepopulation_status import (
    UserListPrepopulationStatusEnum,
)
from .user_list_rule_type import (
    UserListRuleTypeEnum,
)
from .user_list_size_range import (
    UserListSizeRangeEnum,
)
from .user_list_string_rule_item_operator import (
    UserListStringRuleItemOperatorEnum,
)
from .user_list_type import (
    UserListTypeEnum,
)
from .value_rule_device_type import (
    ValueRuleDeviceTypeEnum,
)
from .value_rule_geo_location_match_type import (
    ValueRuleGeoLocationMatchTypeEnum,
)
from .value_rule_operation import (
    ValueRuleOperationEnum,
)
from .value_rule_set_attachment_type import (
    ValueRuleSetAttachmentTypeEnum,
)
from .value_rule_set_dimension import (
    ValueRuleSetDimensionEnum,
)
from .vanity_pharma_display_url_mode import (
    VanityPharmaDisplayUrlModeEnum,
)
from .vanity_pharma_text import (
    VanityPharmaTextEnum,
)
from .video_ad_format_restriction import (
    VideoAdFormatRestrictionEnum,
)
from .video_thumbnail import (
    VideoThumbnailEnum,
)
from .webpage_condition_operand import (
    WebpageConditionOperandEnum,
)
from .webpage_condition_operator import (
    WebpageConditionOperatorEnum,
)

__all__ = (
    "AccessInvitationStatusEnum",
    "AccessReasonEnum",
    "AccessRoleEnum",
    "AccountBudgetProposalStatusEnum",
    "AccountBudgetProposalTypeEnum",
    "AccountBudgetStatusEnum",
    "AccountLinkStatusEnum",
    "AdDestinationTypeEnum",
    "AdFormatTypeEnum",
    "AdGroupAdPrimaryStatusEnum",
    "AdGroupAdPrimaryStatusReasonEnum",
    "AdGroupAdRotationModeEnum",
    "AdGroupAdStatusEnum",
    "AdGroupCriterionApprovalStatusEnum",
    "AdGroupCriterionPrimaryStatusEnum",
    "AdGroupCriterionPrimaryStatusReasonEnum",
    "AdGroupCriterionStatusEnum",
    "AdGroupPrimaryStatusEnum",
    "AdGroupPrimaryStatusReasonEnum",
    "AdGroupStatusEnum",
    "AdGroupTypeEnum",
    "AdNetworkTypeEnum",
    "AdServingOptimizationStatusEnum",
    "AdStrengthEnum",
    "AdStrengthActionItemTypeEnum",
    "AdTypeEnum",
    "AdvertisingChannelSubTypeEnum",
    "AdvertisingChannelTypeEnum",
    "AgeRangeTypeEnum",
    "AndroidPrivacyInteractionTypeEnum",
    "AndroidPrivacyNetworkTypeEnum",
    "AppBiddingGoalEnum",
    "AppCampaignAppStoreEnum",
    "AppCampaignBiddingStrategyGoalTypeEnum",
    "AppPaymentModelTypeEnum",
    "AppUrlOperatingSystemTypeEnum",
    "ApplicationInstanceEnum",
    "AssetAutomationStatusEnum",
    "AssetAutomationTypeEnum",
    "AssetCoverageVideoAspectRatioRequirementEnum",
    "AssetFieldTypeEnum",
    "AssetGroupPrimaryStatusEnum",
    "AssetGroupPrimaryStatusReasonEnum",
    "AssetGroupSignalApprovalStatusEnum",
    "AssetGroupStatusEnum",
    "AssetLinkPrimaryStatusEnum",
    "AssetLinkPrimaryStatusReasonEnum",
    "AssetLinkStatusEnum",
    "AssetOfflineEvaluationErrorReasonsEnum",
    "AssetPerformanceLabelEnum",
    "AssetSetAssetStatusEnum",
    "AssetSetLinkStatusEnum",
    "AssetSetStatusEnum",
    "AssetSetTypeEnum",
    "AssetSourceEnum",
    "AssetTypeEnum",
    "AsyncActionStatusEnum",
    "AttributionModelEnum",
    "AudienceInsightsDimensionEnum",
    "AudienceInsightsMarketingObjectiveEnum",
    "AudienceScopeEnum",
    "AudienceStatusEnum",
    "BatchJobStatusEnum",
    "BidModifierSourceEnum",
    "BiddingSourceEnum",
    "BiddingStrategyStatusEnum",
    "BiddingStrategySystemStatusEnum",
    "BiddingStrategyTypeEnum",
    "BillingSetupStatusEnum",
    "BrandRequestRejectionReasonEnum",
    "BrandSafetySuitabilityEnum",
    "BrandStateEnum",
    "BudgetCampaignAssociationStatusEnum",
    "BudgetDeliveryMethodEnum",
    "BudgetPeriodEnum",
    "BudgetStatusEnum",
    "BudgetTypeEnum",
    "BusinessMessageCallToActionTypeEnum",
    "BusinessMessageProviderEnum",
    "CallConversionReportingStateEnum",
    "CallToActionTypeEnum",
    "CallTrackingDisplayLocationEnum",
    "CallTypeEnum",
    "CampaignCriterionStatusEnum",
    "CampaignDraftStatusEnum",
    "CampaignExperimentTypeEnum",
    "CampaignGroupStatusEnum",
    "CampaignKeywordMatchTypeEnum",
    "CampaignPrimaryStatusEnum",
    "CampaignPrimaryStatusReasonEnum",
    "CampaignServingStatusEnum",
    "CampaignSharedSetStatusEnum",
    "CampaignStatusEnum",
    "ChainRelationshipTypeEnum",
    "ChangeClientTypeEnum",
    "ChangeEventResourceTypeEnum",
    "ChangeStatusOperationEnum",
    "ChangeStatusResourceTypeEnum",
    "ClickTypeEnum",
    "CombinedAudienceStatusEnum",
    "ConsentStatusEnum",
    "ContentLabelTypeEnum",
    "ConversionActionCategoryEnum",
    "ConversionActionCountingTypeEnum",
    "ConversionActionStatusEnum",
    "ConversionActionTypeEnum",
    "ConversionAdjustmentTypeEnum",
    "ConversionAttributionEventTypeEnum",
    "ConversionCustomVariableStatusEnum",
    "ConversionCustomerTypeEnum",
    "ConversionEnvironmentEnum",
    "ConversionLagBucketEnum",
    "ConversionOrAdjustmentLagBucketEnum",
    "ConversionOriginEnum",
    "ConversionTrackingStatusEnum",
    "ConversionValueRulePrimaryDimensionEnum",
    "ConversionValueRuleSetStatusEnum",
    "ConversionValueRuleStatusEnum",
    "ConvertingUserPriorEngagementTypeAndLtvBucketEnum",
    "CriterionCategoryChannelAvailabilityModeEnum",
    "CriterionCategoryLocaleAvailabilityModeEnum",
    "CriterionSystemServingStatusEnum",
    "CriterionTypeEnum",
    "CustomAudienceMemberTypeEnum",
    "CustomAudienceStatusEnum",
    "CustomAudienceTypeEnum",
    "CustomConversionGoalStatusEnum",
    "CustomInterestMemberTypeEnum",
    "CustomInterestStatusEnum",
    "CustomInterestTypeEnum",
    "CustomerAcquisitionOptimizationModeEnum",
    "CustomerMatchUploadKeyTypeEnum",
    "CustomerPayPerConversionEligibilityFailureReasonEnum",
    "CustomerStatusEnum",
    "CustomizerAttributeStatusEnum",
    "CustomizerAttributeTypeEnum",
    "CustomizerValueStatusEnum",
    "DataDrivenModelStatusEnum",
    "DataLinkStatusEnum",
    "DataLinkTypeEnum",
    "DayOfWeekEnum",
    "DemandGenChannelConfigEnum",
    "DemandGenChannelStrategyEnum",
    "DeviceEnum",
    "DisplayAdFormatSettingEnum",
    "DisplayUploadProductTypeEnum",
    "DistanceBucketEnum",
    "EuPoliticalAdvertisingStatusEnum",
    "ExperimentMetricEnum",
    "ExperimentMetricDirectionEnum",
    "ExperimentStatusEnum",
    "ExperimentTypeEnum",
    "ExternalConversionSourceEnum",
    "FixedCpmGoalEnum",
    "FixedCpmTargetFrequencyTimeUnitEnum",
    "FrequencyCapEventTypeEnum",
    "FrequencyCapLevelEnum",
    "FrequencyCapTimeUnitEnum",
    "GenderTypeEnum",
    "GeoTargetConstantStatusEnum",
    "GeoTargetingTypeEnum",
    "GoalConfigLevelEnum",
    "GoogleAdsFieldCategoryEnum",
    "GoogleAdsFieldDataTypeEnum",
    "GoogleVoiceCallStatusEnum",
    "HotelAssetSuggestionStatusEnum",
    "HotelDateSelectionTypeEnum",
    "HotelPriceBucketEnum",
    "HotelRateTypeEnum",
    "HotelReconciliationStatusEnum",
    "IdentityVerificationProgramEnum",
    "IdentityVerificationProgramStatusEnum",
    "IncomeRangeTypeEnum",
    "InsightsKnowledgeGraphEntityCapabilitiesEnum",
    "InsightsTrendEnum",
    "InteractionEventTypeEnum",
    "InteractionTypeEnum",
    "InvoiceTypeEnum",
    "KeywordMatchTypeEnum",
    "KeywordPlanAggregateMetricTypeEnum",
    "KeywordPlanCompetitionLevelEnum",
    "KeywordPlanConceptGroupTypeEnum",
    "KeywordPlanForecastIntervalEnum",
    "KeywordPlanKeywordAnnotationEnum",
    "KeywordPlanNetworkEnum",
    "LabelStatusEnum",
    "LeadFormCallToActionTypeEnum",
    "LeadFormDesiredIntentEnum",
    "LeadFormFieldUserInputTypeEnum",
    "LeadFormPostSubmitCallToActionTypeEnum",
    "LegacyAppInstallAdAppStoreEnum",
    "LinkedAccountTypeEnum",
    "LinkedProductTypeEnum",
    "ListingGroupFilterCustomAttributeIndexEnum",
    "ListingGroupFilterListingSourceEnum",
    "ListingGroupFilterProductCategoryLevelEnum",
    "ListingGroupFilterProductChannelEnum",
    "ListingGroupFilterProductConditionEnum",
    "ListingGroupFilterProductTypeLevelEnum",
    "ListingGroupFilterTypeEnum",
    "ListingGroupTypeEnum",
    "ListingTypeEnum",
    "LocalServicesBusinessRegistrationCheckRejectionReasonEnum",
    "LocalServicesBusinessRegistrationTypeEnum",
    "LocalServicesLeadConversationTypeEnum",
    "LocalServicesEmployeeStatusEnum",
    "LocalServicesEmployeeTypeEnum",
    "LocalServicesInsuranceRejectionReasonEnum",
    "LocalServicesLeadCreditIssuanceDecisionEnum",
    "LocalServicesCreditStateEnum",
    "LocalServicesLeadStatusEnum",
    "LocalServicesLeadSurveyAnswerEnum",
    "LocalServicesLeadSurveyDissatisfiedReasonEnum",
    "LocalServicesLeadSurveySatisfiedReasonEnum",
    "LocalServicesLeadTypeEnum",
    "LocalServicesLicenseRejectionReasonEnum",
    "LocalServicesParticipantTypeEnum",
    "LocalServicesVerificationArtifactStatusEnum",
    "LocalServicesVerificationArtifactTypeEnum",
    "LocalServicesVerificationStatusEnum",
    "LocationGroupRadiusUnitsEnum",
    "LocationOwnershipTypeEnum",
    "LocationSourceTypeEnum",
    "LocationStringFilterTypeEnum",
    "LookalikeExpansionLevelEnum",
    "ManagerLinkStatusEnum",
    "MediaTypeEnum",
    "MimeTypeEnum",
    "MinuteOfHourEnum",
    "MobileAppVendorEnum",
    "MobileDeviceTypeEnum",
    "MonthOfYearEnum",
    "NegativeGeoTargetTypeEnum",
    "NonSkippableMaxDurationEnum",
    "NonSkippableMinDurationEnum",
    "OfflineConversionDiagnosticStatusEnum",
    "OfflineEventUploadClientEnum",
    "OfflineUserDataJobFailureReasonEnum",
    "OfflineUserDataJobMatchRateRangeEnum",
    "OfflineUserDataJobStatusEnum",
    "OfflineUserDataJobTypeEnum",
    "OperatingSystemVersionOperatorTypeEnum",
    "OptimizationGoalTypeEnum",
    "ParentalStatusTypeEnum",
    "PaymentModeEnum",
    "PerformanceMaxUpgradeStatusEnum",
    "PlacementTypeEnum",
    "PolicyApprovalStatusEnum",
    "PolicyReviewStatusEnum",
    "PolicyTopicEntryTypeEnum",
    "PolicyTopicEvidenceDestinationMismatchUrlTypeEnum",
    "PolicyTopicEvidenceDestinationNotWorkingDeviceEnum",
    "PolicyTopicEvidenceDestinationNotWorkingDnsErrorTypeEnum",
    "PositiveGeoTargetTypeEnum",
    "PriceExtensionPriceQualifierEnum",
    "PriceExtensionPriceUnitEnum",
    "PriceExtensionTypeEnum",
    "ProductAvailabilityEnum",
    "ProductCategoryLevelEnum",
    "ProductCategoryStateEnum",
    "ProductChannelEnum",
    "ProductChannelExclusivityEnum",
    "ProductConditionEnum",
    "ProductCustomAttributeIndexEnum",
    "ProductIssueSeverityEnum",
    "ProductLinkInvitationStatusEnum",
    "ProductStatusEnum",
    "ProductTypeLevelEnum",
    "PromotionExtensionDiscountModifierEnum",
    "PromotionExtensionOccasionEnum",
    "ProximityRadiusUnitsEnum",
    "QualityScoreBucketEnum",
    "ReachPlanAgeRangeEnum",
    "ReachPlanConversionRateModelEnum",
    "ReachPlanNetworkEnum",
    "ReachPlanPlannableUserListStatusEnum",
    "ReachPlanSurfaceEnum",
    "RecommendationSubscriptionStatusEnum",
    "RecommendationTypeEnum",
    "ResourceChangeOperationEnum",
    "ResourceLimitTypeEnum",
    "ResponseContentTypeEnum",
    "SearchEngineResultsPageTypeEnum",
    "SearchTermMatchTypeEnum",
    "SearchTermTargetingStatusEnum",
    "SeasonalityEventScopeEnum",
    "SeasonalityEventStatusEnum",
    "ServedAssetFieldTypeEnum",
    "SharedSetStatusEnum",
    "SharedSetTypeEnum",
    "ShoppingAddProductsToCampaignRecommendationEnum",
    "SimulationModificationMethodEnum",
    "SimulationTypeEnum",
    "SkAdNetworkAdEventTypeEnum",
    "SkAdNetworkAttributionCreditEnum",
    "SkAdNetworkCoarseConversionValueEnum",
    "SkAdNetworkSourceTypeEnum",
    "SkAdNetworkUserTypeEnum",
    "SlotEnum",
    "SmartCampaignNotEligibleReasonEnum",
    "SmartCampaignStatusEnum",
    "SpendingLimitTypeEnum",
    "SummaryRowSettingEnum",
    "SystemManagedResourceSourceEnum",
    "TargetCpaOptInRecommendationGoalEnum",
    "TargetFrequencyTimeUnitEnum",
    "TargetImpressionShareLocationEnum",
    "TargetingDimensionEnum",
    "TimeTypeEnum",
    "TrackingCodePageFormatEnum",
    "TrackingCodeTypeEnum",
    "UserIdentifierSourceEnum",
    "UserInterestTaxonomyTypeEnum",
    "UserListAccessStatusEnum",
    "UserListClosingReasonEnum",
    "UserListCrmDataSourceTypeEnum",
    "UserListCustomerTypeCategoryEnum",
    "UserListDateRuleItemOperatorEnum",
    "UserListFlexibleRuleOperatorEnum",
    "UserListLogicalRuleOperatorEnum",
    "UserListMembershipStatusEnum",
    "UserListNumberRuleItemOperatorEnum",
    "UserListPrepopulationStatusEnum",
    "UserListRuleTypeEnum",
    "UserListSizeRangeEnum",
    "UserListStringRuleItemOperatorEnum",
    "UserListTypeEnum",
    "ValueRuleDeviceTypeEnum",
    "ValueRuleGeoLocationMatchTypeEnum",
    "ValueRuleOperationEnum",
    "ValueRuleSetAttachmentTypeEnum",
    "ValueRuleSetDimensionEnum",
    "VanityPharmaDisplayUrlModeEnum",
    "VanityPharmaTextEnum",
    "VideoAdFormatRestrictionEnum",
    "VideoThumbnailEnum",
    "WebpageConditionOperandEnum",
    "WebpageConditionOperatorEnum",
)



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example adds a hotel campaign, ad group, and ad group ad.

Prerequisite: You need to have access to the Hotel Ads Center, which can be
granted during integration with Google Hotels. The integration instructions can
be found at:
https://support.google.com/hotelprices/answer/6101897.
"""


import argparse
import sys
import uuid

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group import AdGroup
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_budget_service import (
    CampaignBudgetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
    MutateAdGroupAdsResponse,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
    MutateAdGroupsResponse,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
    MutateCampaignBudgetsResponse,
)
from google.ads.googleads.v22.services.types.campaign_service import (
    CampaignOperation,
    MutateCampaignsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    hotel_center_account_id: int,
    cpc_bid_ceiling_micro_amount: int,
) -> None:
    budget_resource_name: str = add_budget(client, customer_id)

    campaign_resource_name: str = add_hotel_campaign(
        client,
        customer_id,
        budget_resource_name,
        hotel_center_account_id,
        cpc_bid_ceiling_micro_amount,
    )

    ad_group_resource_name: str = add_hotel_ad_group(
        client, customer_id, campaign_resource_name
    )

    add_hotel_ad(client, customer_id, ad_group_resource_name)


def add_budget(client: GoogleAdsClient, customer_id: str) -> str:
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )

    # Create a budget, which can be shared by multiple campaigns.
    campaign_budget_operation: CampaignBudgetOperation = client.get_type(
        "CampaignBudgetOperation"
    )
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Interplanetary Budget {uuid.uuid4()}"
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    campaign_budget.amount_micros = 500000

    # Add budget.
    campaign_budget_response: MutateCampaignBudgetsResponse = (
        campaign_budget_service.mutate_campaign_budgets(
            customer_id=customer_id, operations=[campaign_budget_operation]
        )
    )

    budget_resource_name: str = campaign_budget_response.results[
        0
    ].resource_name

    print(f"Created budget with resource name '{budget_resource_name}'.")

    return budget_resource_name


# [START add_hotel_ad_3]
def add_hotel_ad(
    client: GoogleAdsClient, customer_id: str, ad_group_resource_name: str
) -> str:
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )

    # Creates a new ad group ad and sets the hotel ad to it.
    ad_group_ad_operation: AdGroupAdOperation = client.get_type(
        "AdGroupAdOperation"
    )
    ad_group_ad: AdGroupAd = ad_group_ad_operation.create
    ad_group_ad.ad_group = ad_group_resource_name
    # Set the ad group ad to enabled.  Setting this to paused will cause an error
    # for hotel campaigns.  For hotels pausing should happen at either the ad group or
    # campaign level.
    ad_group_ad.status = client.enums.AdGroupAdStatusEnum.ENABLED
    client.copy_from(ad_group_ad.ad.hotel_ad, client.get_type("HotelAdInfo"))

    # Add the ad group ad.
    ad_group_ad_response: MutateAdGroupAdsResponse = (
        ad_group_ad_service.mutate_ad_group_ads(
            customer_id=customer_id, operations=[ad_group_ad_operation]
        )
    )

    ad_group_ad_resource_name: str = ad_group_ad_response.results[
        0
    ].resource_name

    print(f"Created hotel ad with resource name '{ad_group_ad_resource_name}'.")

    return ad_group_resource_name
    # [END add_hotel_ad_3]


# [START add_hotel_ad_2]
def add_hotel_ad_group(
    client: GoogleAdsClient, customer_id: str, campaign_resource_name: str
) -> str:
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )

    # Create ad group.
    ad_group_operation: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group: AdGroup = ad_group_operation.create
    ad_group.name = f"Earth to Mars cruise {uuid.uuid4()}"
    ad_group.status = client.enums.AdGroupStatusEnum.ENABLED
    ad_group.campaign = campaign_resource_name
    # Sets the ad group type to HOTEL_ADS. This cannot be set to other types.
    ad_group.type_ = client.enums.AdGroupTypeEnum.HOTEL_ADS
    ad_group.cpc_bid_micros = 10000000

    # Add the ad group.
    ad_group_response: MutateAdGroupsResponse = (
        ad_group_service.mutate_ad_groups(
            customer_id=customer_id, operations=[ad_group_operation]
        )
    )

    ad_group_resource_name: str = ad_group_response.results[0].resource_name

    print(
        "Added a hotel ad group with resource name '{ad_group_resource_name}'."
    )

    return ad_group_resource_name
    # [END add_hotel_ad_2]


# [START add_hotel_ad]
def add_hotel_campaign(
    client: GoogleAdsClient,
    customer_id: str,
    budget_resource_name: str,
    hotel_center_account_id: int,
    cpc_bid_ceiling_micro_amount: int,
) -> str:
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )

    # [START add_hotel_ad_1]
    # Create campaign.
    campaign_operation: CampaignOperation = client.get_type("CampaignOperation")
    campaign: Campaign = campaign_operation.create
    campaign.name = f"Interplanetary Cruise Campaign {uuid.uuid4()}"

    # Configures settings related to hotel campaigns including advertising
    # channel type and hotel setting info.
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.HOTEL
    )
    campaign.hotel_setting.hotel_center_id = hotel_center_account_id

    # Recommendation: Set the campaign to PAUSED when creating it to prevent the
    # ads from immediately serving. Set to ENABLED once you've added targeting
    # and the ads are ready to serve.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED

    # Set the bidding strategy to PercentCpc. Only Manual CPC and Percent CPC
    # can be used for hotel campaigns.
    campaign.percent_cpc.cpc_bid_ceiling_micros = cpc_bid_ceiling_micro_amount

    # Sets the budget.
    campaign.campaign_budget = budget_resource_name

    # Set the campaign network options. Only Google Search is allowed for hotel
    # campaigns.
    campaign.network_settings.target_google_search = True

    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )
    # [END add_hotel_ad_1]

    # Add the campaign.
    campaign_response: MutateCampaignsResponse = (
        campaign_service.mutate_campaigns(
            customer_id=customer_id, operations=[campaign_operation]
        )
    )

    campaign_resource_name: str = campaign_response.results[0].resource_name

    print(
        "Added a hotel campaign with resource name '{campaign_resource_name}'."
    )

    return campaign_resource_name
    # [END add_hotel_ad]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=(
            "Adds an expanded text ad to the specified ad group ID, "
            "for the given customer ID."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-b",
        "--cpc_bid_ceiling_micro_amount",
        type=int,
        required=True,
        help=("The cpc bid ceiling micro amount for the hotel campaign."),
    )
    parser.add_argument(
        "-a",
        "--hotel_center_account_id",
        type=int,
        required=True,
        help="The hotel center account ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.hotel_center_account_id,
            args.cpc_bid_ceiling_micro_amount,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "ManagedPlacementView",
    },
)


class ManagedPlacementView(proto.Message):
    r"""A managed placement view.

    Attributes:
        resource_name (str):
            Output only. The resource name of the Managed Placement
            view. Managed placement view resource names have the form:

            ``customers/{customer_id}/managedPlacementViews/{ad_group_id}~{criterion_id}``
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .access_invitation_error import (
    AccessInvitationErrorEnum,
)
from .account_budget_proposal_error import (
    AccountBudgetProposalErrorEnum,
)
from .account_link_error import (
    AccountLinkErrorEnum,
)
from .ad_customizer_error import (
    AdCustomizerErrorEnum,
)
from .ad_error import (
    AdErrorEnum,
)
from .ad_group_ad_error import (
    AdGroupAdErrorEnum,
)
from .ad_group_bid_modifier_error import (
    AdGroupBidModifierErrorEnum,
)
from .ad_group_criterion_customizer_error import (
    AdGroupCriterionCustomizerErrorEnum,
)
from .ad_group_criterion_error import (
    AdGroupCriterionErrorEnum,
)
from .ad_group_customizer_error import (
    AdGroupCustomizerErrorEnum,
)
from .ad_group_error import (
    AdGroupErrorEnum,
)
from .ad_group_feed_error import (
    AdGroupFeedErrorEnum,
)
from .ad_parameter_error import (
    AdParameterErrorEnum,
)
from .ad_sharing_error import (
    AdSharingErrorEnum,
)
from .adx_error import (
    AdxErrorEnum,
)
from .asset_error import (
    AssetErrorEnum,
)
from .asset_group_asset_error import (
    AssetGroupAssetErrorEnum,
)
from .asset_group_error import (
    AssetGroupErrorEnum,
)
from .asset_group_listing_group_filter_error import (
    AssetGroupListingGroupFilterErrorEnum,
)
from .asset_group_signal_error import (
    AssetGroupSignalErrorEnum,
)
from .asset_link_error import (
    AssetLinkErrorEnum,
)
from .asset_set_asset_error import (
    AssetSetAssetErrorEnum,
)
from .asset_set_error import (
    AssetSetErrorEnum,
)
from .asset_set_link_error import (
    AssetSetLinkErrorEnum,
)
from .audience_error import (
    AudienceErrorEnum,
)
from .audience_insights_error import (
    AudienceInsightsErrorEnum,
)
from .authentication_error import (
    AuthenticationErrorEnum,
)
from .authorization_error import (
    AuthorizationErrorEnum,
)
from .automatically_created_asset_removal_error import (
    AutomaticallyCreatedAssetRemovalErrorEnum,
)
from .batch_job_error import (
    BatchJobErrorEnum,
)
from .bidding_error import (
    BiddingErrorEnum,
)
from .bidding_strategy_error import (
    BiddingStrategyErrorEnum,
)
from .billing_setup_error import (
    BillingSetupErrorEnum,
)
from .brand_guidelines_migration_error import (
    BrandGuidelinesMigrationErrorEnum,
)
from .campaign_budget_error import (
    CampaignBudgetErrorEnum,
)
from .campaign_conversion_goal_error import (
    CampaignConversionGoalErrorEnum,
)
from .campaign_criterion_error import (
    CampaignCriterionErrorEnum,
)
from .campaign_customizer_error import (
    CampaignCustomizerErrorEnum,
)
from .campaign_draft_error import (
    CampaignDraftErrorEnum,
)
from .campaign_error import (
    CampaignErrorEnum,
)
from .campaign_experiment_error import (
    CampaignExperimentErrorEnum,
)
from .campaign_feed_error import (
    CampaignFeedErrorEnum,
)
from .campaign_lifecycle_goal_error import (
    CampaignLifecycleGoalErrorEnum,
)
from .campaign_shared_set_error import (
    CampaignSharedSetErrorEnum,
)
from .change_event_error import (
    ChangeEventErrorEnum,
)
from .change_status_error import (
    ChangeStatusErrorEnum,
)
from .collection_size_error import (
    CollectionSizeErrorEnum,
)
from .context_error import (
    ContextErrorEnum,
)
from .conversion_action_error import (
    ConversionActionErrorEnum,
)
from .conversion_adjustment_upload_error import (
    ConversionAdjustmentUploadErrorEnum,
)
from .conversion_custom_variable_error import (
    ConversionCustomVariableErrorEnum,
)
from .conversion_goal_campaign_config_error import (
    ConversionGoalCampaignConfigErrorEnum,
)
from .conversion_upload_error import (
    ConversionUploadErrorEnum,
)
from .conversion_value_rule_error import (
    ConversionValueRuleErrorEnum,
)
from .conversion_value_rule_set_error import (
    ConversionValueRuleSetErrorEnum,
)
from .country_code_error import (
    CountryCodeErrorEnum,
)
from .criterion_error import (
    CriterionErrorEnum,
)
from .currency_code_error import (
    CurrencyCodeErrorEnum,
)
from .currency_error import (
    CurrencyErrorEnum,
)
from .custom_audience_error import (
    CustomAudienceErrorEnum,
)
from .custom_conversion_goal_error import (
    CustomConversionGoalErrorEnum,
)
from .custom_interest_error import (
    CustomInterestErrorEnum,
)
from .customer_client_link_error import (
    CustomerClientLinkErrorEnum,
)
from .customer_customizer_error import (
    CustomerCustomizerErrorEnum,
)
from .customer_error import (
    CustomerErrorEnum,
)
from .customer_feed_error import (
    CustomerFeedErrorEnum,
)
from .customer_lifecycle_goal_error import (
    CustomerLifecycleGoalErrorEnum,
)
from .customer_manager_link_error import (
    CustomerManagerLinkErrorEnum,
)
from .customer_sk_ad_network_conversion_value_schema_error import (
    CustomerSkAdNetworkConversionValueSchemaErrorEnum,
)
from .customer_user_access_error import (
    CustomerUserAccessErrorEnum,
)
from .customizer_attribute_error import (
    CustomizerAttributeErrorEnum,
)
from .data_link_error import (
    DataLinkErrorEnum,
)
from .database_error import (
    DatabaseErrorEnum,
)
from .date_error import (
    DateErrorEnum,
)
from .date_range_error import (
    DateRangeErrorEnum,
)
from .distinct_error import (
    DistinctErrorEnum,
)
from .enum_error import (
    EnumErrorEnum,
)
from .errors import (
    ErrorCode,
    ErrorDetails,
    ErrorLocation,
    GoogleAdsError,
    GoogleAdsFailure,
    PolicyFindingDetails,
    PolicyViolationDetails,
    QuotaErrorDetails,
    ResourceCountDetails,
)
from .experiment_arm_error import (
    ExperimentArmErrorEnum,
)
from .experiment_error import (
    ExperimentErrorEnum,
)
from .extension_feed_item_error import (
    ExtensionFeedItemErrorEnum,
)
from .extension_setting_error import (
    ExtensionSettingErrorEnum,
)
from .feed_attribute_reference_error import (
    FeedAttributeReferenceErrorEnum,
)
from .feed_error import (
    FeedErrorEnum,
)
from .feed_item_error import (
    FeedItemErrorEnum,
)
from .feed_item_set_error import (
    FeedItemSetErrorEnum,
)
from .feed_item_set_link_error import (
    FeedItemSetLinkErrorEnum,
)
from .feed_item_target_error import (
    FeedItemTargetErrorEnum,
)
from .feed_item_validation_error import (
    FeedItemValidationErrorEnum,
)
from .feed_mapping_error import (
    FeedMappingErrorEnum,
)
from .field_error import (
    FieldErrorEnum,
)
from .field_mask_error import (
    FieldMaskErrorEnum,
)
from .function_error import (
    FunctionErrorEnum,
)
from .function_parsing_error import (
    FunctionParsingErrorEnum,
)
from .geo_target_constant_suggestion_error import (
    GeoTargetConstantSuggestionErrorEnum,
)
from .header_error import (
    HeaderErrorEnum,
)
from .id_error import (
    IdErrorEnum,
)
from .identity_verification_error import (
    IdentityVerificationErrorEnum,
)
from .image_error import (
    ImageErrorEnum,
)
from .internal_error import (
    InternalErrorEnum,
)
from .invoice_error import (
    InvoiceErrorEnum,
)
from .keyword_plan_ad_group_error import (
    KeywordPlanAdGroupErrorEnum,
)
from .keyword_plan_ad_group_keyword_error import (
    KeywordPlanAdGroupKeywordErrorEnum,
)
from .keyword_plan_campaign_error import (
    KeywordPlanCampaignErrorEnum,
)
from .keyword_plan_campaign_keyword_error import (
    KeywordPlanCampaignKeywordErrorEnum,
)
from .keyword_plan_error import (
    KeywordPlanErrorEnum,
)
from .keyword_plan_idea_error import (
    KeywordPlanIdeaErrorEnum,
)
from .label_error import (
    LabelErrorEnum,
)
from .language_code_error import (
    LanguageCodeErrorEnum,
)
from .list_operation_error import (
    ListOperationErrorEnum,
)
from .manager_link_error import (
    ManagerLinkErrorEnum,
)
from .media_bundle_error import (
    MediaBundleErrorEnum,
)
from .media_file_error import (
    MediaFileErrorEnum,
)
from .media_upload_error import (
    MediaUploadErrorEnum,
)
from .merchant_center_error import (
    MerchantCenterErrorEnum,
)
from .multiplier_error import (
    MultiplierErrorEnum,
)
from .mutate_error import (
    MutateErrorEnum,
)
from .new_resource_creation_error import (
    NewResourceCreationErrorEnum,
)
from .not_allowlisted_error import (
    NotAllowlistedErrorEnum,
)
from .not_empty_error import (
    NotEmptyErrorEnum,
)
from .null_error import (
    NullErrorEnum,
)
from .offline_user_data_job_error import (
    OfflineUserDataJobErrorEnum,
)
from .operation_access_denied_error import (
    OperationAccessDeniedErrorEnum,
)
from .operator_error import (
    OperatorErrorEnum,
)
from .partial_failure_error import (
    PartialFailureErrorEnum,
)
from .payments_account_error import (
    PaymentsAccountErrorEnum,
)
from .policy_finding_error import (
    PolicyFindingErrorEnum,
)
from .policy_validation_parameter_error import (
    PolicyValidationParameterErrorEnum,
)
from .policy_violation_error import (
    PolicyViolationErrorEnum,
)
from .product_link_error import (
    ProductLinkErrorEnum,
)
from .product_link_invitation_error import (
    ProductLinkInvitationErrorEnum,
)
from .query_error import (
    QueryErrorEnum,
)
from .quota_error import (
    QuotaErrorEnum,
)
from .range_error import (
    RangeErrorEnum,
)
from .reach_plan_error import (
    ReachPlanErrorEnum,
)
from .recommendation_error import (
    RecommendationErrorEnum,
)
from .recommendation_subscription_error import (
    RecommendationSubscriptionErrorEnum,
)
from .region_code_error import (
    RegionCodeErrorEnum,
)
from .request_error import (
    RequestErrorEnum,
)
from .resource_access_denied_error import (
    ResourceAccessDeniedErrorEnum,
)
from .resource_count_limit_exceeded_error import (
    ResourceCountLimitExceededErrorEnum,
)
from .search_term_insight_error import (
    SearchTermInsightErrorEnum,
)
from .setting_error import (
    SettingErrorEnum,
)
from .shareable_preview_error import (
    ShareablePreviewErrorEnum,
)
from .shared_criterion_error import (
    SharedCriterionErrorEnum,
)
from .shared_set_error import (
    SharedSetErrorEnum,
)
from .shopping_product_error import (
    ShoppingProductErrorEnum,
)
from .size_limit_error import (
    SizeLimitErrorEnum,
)
from .smart_campaign_error import (
    SmartCampaignErrorEnum,
)
from .string_format_error import (
    StringFormatErrorEnum,
)
from .string_length_error import (
    StringLengthErrorEnum,
)
from .third_party_app_analytics_link_error import (
    ThirdPartyAppAnalyticsLinkErrorEnum,
)
from .time_zone_error import (
    TimeZoneErrorEnum,
)
from .url_field_error import (
    UrlFieldErrorEnum,
)
from .user_data_error import (
    UserDataErrorEnum,
)
from .user_list_customer_type_error import (
    UserListCustomerTypeErrorEnum,
)
from .user_list_error import (
    UserListErrorEnum,
)
from .video_campaign_error import (
    VideoCampaignErrorEnum,
)
from .youtube_video_registration_error import (
    YoutubeVideoRegistrationErrorEnum,
)

__all__ = (
    "AccessInvitationErrorEnum",
    "AccountBudgetProposalErrorEnum",
    "AccountLinkErrorEnum",
    "AdCustomizerErrorEnum",
    "AdErrorEnum",
    "AdGroupAdErrorEnum",
    "AdGroupBidModifierErrorEnum",
    "AdGroupCriterionCustomizerErrorEnum",
    "AdGroupCriterionErrorEnum",
    "AdGroupCustomizerErrorEnum",
    "AdGroupErrorEnum",
    "AdGroupFeedErrorEnum",
    "AdParameterErrorEnum",
    "AdSharingErrorEnum",
    "AdxErrorEnum",
    "AssetErrorEnum",
    "AssetGroupAssetErrorEnum",
    "AssetGroupErrorEnum",
    "AssetGroupListingGroupFilterErrorEnum",
    "AssetGroupSignalErrorEnum",
    "AssetLinkErrorEnum",
    "AssetSetAssetErrorEnum",
    "AssetSetErrorEnum",
    "AssetSetLinkErrorEnum",
    "AudienceErrorEnum",
    "AudienceInsightsErrorEnum",
    "AuthenticationErrorEnum",
    "AuthorizationErrorEnum",
    "AutomaticallyCreatedAssetRemovalErrorEnum",
    "BatchJobErrorEnum",
    "BiddingErrorEnum",
    "BiddingStrategyErrorEnum",
    "BillingSetupErrorEnum",
    "BrandGuidelinesMigrationErrorEnum",
    "CampaignBudgetErrorEnum",
    "CampaignConversionGoalErrorEnum",
    "CampaignCriterionErrorEnum",
    "CampaignCustomizerErrorEnum",
    "CampaignDraftErrorEnum",
    "CampaignErrorEnum",
    "CampaignExperimentErrorEnum",
    "CampaignFeedErrorEnum",
    "CampaignLifecycleGoalErrorEnum",
    "CampaignSharedSetErrorEnum",
    "ChangeEventErrorEnum",
    "ChangeStatusErrorEnum",
    "CollectionSizeErrorEnum",
    "ContextErrorEnum",
    "ConversionActionErrorEnum",
    "ConversionAdjustmentUploadErrorEnum",
    "ConversionCustomVariableErrorEnum",
    "ConversionGoalCampaignConfigErrorEnum",
    "ConversionUploadErrorEnum",
    "ConversionValueRuleErrorEnum",
    "ConversionValueRuleSetErrorEnum",
    "CountryCodeErrorEnum",
    "CriterionErrorEnum",
    "CurrencyCodeErrorEnum",
    "CurrencyErrorEnum",
    "CustomAudienceErrorEnum",
    "CustomConversionGoalErrorEnum",
    "CustomInterestErrorEnum",
    "CustomerClientLinkErrorEnum",
    "CustomerCustomizerErrorEnum",
    "CustomerErrorEnum",
    "CustomerFeedErrorEnum",
    "CustomerLifecycleGoalErrorEnum",
    "CustomerManagerLinkErrorEnum",
    "CustomerSkAdNetworkConversionValueSchemaErrorEnum",
    "CustomerUserAccessErrorEnum",
    "CustomizerAttributeErrorEnum",
    "DataLinkErrorEnum",
    "DatabaseErrorEnum",
    "DateErrorEnum",
    "DateRangeErrorEnum",
    "DistinctErrorEnum",
    "EnumErrorEnum",
    "ErrorCode",
    "ErrorDetails",
    "ErrorLocation",
    "GoogleAdsError",
    "GoogleAdsFailure",
    "PolicyFindingDetails",
    "PolicyViolationDetails",
    "QuotaErrorDetails",
    "ResourceCountDetails",
    "ExperimentArmErrorEnum",
    "ExperimentErrorEnum",
    "ExtensionFeedItemErrorEnum",
    "ExtensionSettingErrorEnum",
    "FeedAttributeReferenceErrorEnum",
    "FeedErrorEnum",
    "FeedItemErrorEnum",
    "FeedItemSetErrorEnum",
    "FeedItemSetLinkErrorEnum",
    "FeedItemTargetErrorEnum",
    "FeedItemValidationErrorEnum",
    "FeedMappingErrorEnum",
    "FieldErrorEnum",
    "FieldMaskErrorEnum",
    "FunctionErrorEnum",
    "FunctionParsingErrorEnum",
    "GeoTargetConstantSuggestionErrorEnum",
    "HeaderErrorEnum",
    "IdErrorEnum",
    "IdentityVerificationErrorEnum",
    "ImageErrorEnum",
    "InternalErrorEnum",
    "InvoiceErrorEnum",
    "KeywordPlanAdGroupErrorEnum",
    "KeywordPlanAdGroupKeywordErrorEnum",
    "KeywordPlanCampaignErrorEnum",
    "KeywordPlanCampaignKeywordErrorEnum",
    "KeywordPlanErrorEnum",
    "KeywordPlanIdeaErrorEnum",
    "LabelErrorEnum",
    "LanguageCodeErrorEnum",
    "ListOperationErrorEnum",
    "ManagerLinkErrorEnum",
    "MediaBundleErrorEnum",
    "MediaFileErrorEnum",
    "MediaUploadErrorEnum",
    "MerchantCenterErrorEnum",
    "MultiplierErrorEnum",
    "MutateErrorEnum",
    "NewResourceCreationErrorEnum",
    "NotAllowlistedErrorEnum",
    "NotEmptyErrorEnum",
    "NullErrorEnum",
    "OfflineUserDataJobErrorEnum",
    "OperationAccessDeniedErrorEnum",
    "OperatorErrorEnum",
    "PartialFailureErrorEnum",
    "PaymentsAccountErrorEnum",
    "PolicyFindingErrorEnum",
    "PolicyValidationParameterErrorEnum",
    "PolicyViolationErrorEnum",
    "ProductLinkErrorEnum",
    "ProductLinkInvitationErrorEnum",
    "QueryErrorEnum",
    "QuotaErrorEnum",
    "RangeErrorEnum",
    "ReachPlanErrorEnum",
    "RecommendationErrorEnum",
    "RecommendationSubscriptionErrorEnum",
    "RegionCodeErrorEnum",
    "RequestErrorEnum",
    "ResourceAccessDeniedErrorEnum",
    "ResourceCountLimitExceededErrorEnum",
    "SearchTermInsightErrorEnum",
    "SettingErrorEnum",
    "ShareablePreviewErrorEnum",
    "SharedCriterionErrorEnum",
    "SharedSetErrorEnum",
    "ShoppingProductErrorEnum",
    "SizeLimitErrorEnum",
    "SmartCampaignErrorEnum",
    "StringFormatErrorEnum",
    "StringLengthErrorEnum",
    "ThirdPartyAppAnalyticsLinkErrorEnum",
    "TimeZoneErrorEnum",
    "UrlFieldErrorEnum",
    "UserDataErrorEnum",
    "UserListCustomerTypeErrorEnum",
    "UserListErrorEnum",
    "VideoCampaignErrorEnum",
    "YoutubeVideoRegistrationErrorEnum",
)



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This shows how to handle responses that may include partial_failure errors."""


import argparse
import sys
import uuid
from typing import Any, List

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
    MutateAdGroupsResponse,
    MutateAdGroupsRequest,
)


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    """Runs the example code, which demonstrates how to handle partial failures.

    The example creates three Ad Groups, two of which intentionally fail in
    order to generate a partial failure error. It also demonstrates how to
    properly identify a partial error and how to log the error messages.

    Args:
        client:  An initialized GoogleAdsClient instance.
        customer_id: A valid customer account ID.
        campaign_id: The ID for a campaign to create Ad Groups under.
    """
    try:
        ad_group_response: MutateAdGroupsResponse = create_ad_groups(
            client, customer_id, campaign_id
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)
    else:
        print_results(client, ad_group_response)


# [START handle_partial_failure]
def create_ad_groups(
    client: GoogleAdsClient, customer_id: str, campaign_id: str
) -> MutateAdGroupsResponse:
    """Creates three Ad Groups, two of which intentionally generate errors.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: A valid customer account ID.
        campaign_id: The ID for a campaign to create Ad Groups under.

    Returns: A MutateAdGroupsResponse message instance.
    """
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    resource_name: str = campaign_service.campaign_path(
        customer_id, campaign_id
    )

    invalid_resource_name: str = campaign_service.campaign_path(customer_id, 0)
    ad_group_operations: List[AdGroupOperation] = []

    # This AdGroup should be created successfully - assuming the campaign in
    # the params exists.
    ad_group_op1: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group_op1.create.name = f"Valid AdGroup: {uuid.uuid4()}"
    ad_group_op1.create.campaign = resource_name
    ad_group_operations.append(ad_group_op1)

    # This AdGroup will always fail - campaign ID 0 in resource names is
    # never valid.
    ad_group_op2: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group_op2.create.name = f"Broken AdGroup: {uuid.uuid4()}"
    ad_group_op2.create.campaign = invalid_resource_name
    ad_group_operations.append(ad_group_op2)

    # This AdGroup will always fail - duplicate ad group names are not allowed.
    ad_group_op3: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group_op3.create.name = ad_group_op1.create.name
    ad_group_op3.create.campaign = resource_name
    ad_group_operations.append(ad_group_op3)

    # Issue a mutate request, setting partial_failure=True.
    request: MutateAdGroupsRequest = client.get_type("MutateAdGroupsRequest")
    request.customer_id = customer_id
    request.operations = ad_group_operations
    request.partial_failure = True
    return ad_group_service.mutate_ad_groups(request=request)
    # [END handle_partial_failure]


# [START handle_partial_failure_1]
def is_partial_failure_error_present(response: MutateAdGroupsResponse) -> bool:
    """Checks whether a response message has a partial failure error.

    In Python the partial_failure_error attr is always present on a response
    message and is represented by a google.rpc.Status message. So we can't
    simply check whether the field is present, we must check that the code is
    non-zero. Error codes are represented by the google.rpc.Code proto Enum:
    https://github.com/googleapis/googleapis/blob/master/google/rpc/code.proto

    Args:
        response:  A MutateAdGroupsResponse message instance.

    Returns: A boolean, whether or not the response message has a partial
        failure error.
    """
    partial_failure: Any = getattr(response, "partial_failure_error", None)
    code: int = int(getattr(partial_failure, "code", 0))  # Default to 0 if None
    return code != 0
    # [END handle_partial_failure_1]


# [START handle_partial_failure_2]
def print_results(
    client: GoogleAdsClient, response: MutateAdGroupsResponse
) -> None:
    """Prints partial failure errors and success messages from a response.

    This function shows how to retrieve partial_failure errors from a response
    message (in the case of this example the message will be of type
    MutateAdGroupsResponse) and how to unpack those errors to GoogleAdsFailure
    instances. It also shows that a response with partial failures may still
    contain successful requests, and that those messages should be parsed
    separately. As an example, a GoogleAdsFailure object from this example will
    be structured similar to:

    error_code {
      range_error: TOO_LOW
    }
    message: "Too low."
    trigger {
      string_value: ""
    }
    location {
      field_path_elements {
        field_name: "operations"
        index {
          value: 1
        }
      }
      field_path_elements {
        field_name: "create"
      }
      field_path_elements {
        field_name: "campaign"
      }
    }

    Args:
        client: an initialized GoogleAdsClient.
        response: a MutateAdGroupsResponse instance.
    """
    # Check for existence of any partial failures in the response.
    if is_partial_failure_error_present(response):
        print("Partial failures occurred. Details will be shown below.\n")
        # Prints the details of the partial failure errors.
        partial_failure: Any = getattr(response, "partial_failure_error", None)
        # partial_failure_error.details is a repeated field and iterable
        error_details: List[Any] = getattr(partial_failure, "details", [])

        for error_detail in error_details:
            # Retrieve an instance of the GoogleAdsFailure class from the client
            failure_message: Any = client.get_type("GoogleAdsFailure")
            # Parse the string into a GoogleAdsFailure message instance.
            # To access class-only methods on the message we retrieve its type.
            GoogleAdsFailure: Any = type(failure_message)
            failure_object: Any = GoogleAdsFailure.deserialize(
                error_detail.value
            )

            for error in failure_object.errors:
                # Construct and print a string that details which element in
                # the above ad_group_operations list failed (by index number)
                # as well as the error message and error code.
                print(
                    "A partial failure at index "
                    f"{error.location.field_path_elements[0].index} occurred "
                    f"\nError message: {error.message}\nError code: "
                    f"{error.error_code}"
                )
    else:
        print(
            "All operations completed successfully. No partial failure "
            "to show."
        )

    # In the list of results, operations from the ad_group_operation list
    # that failed will be represented as empty messages. This loop detects
    # such empty messages and ignores them, while printing information about
    # successful operations.
    for message in response.results:
        if not message:
            continue

        print(f"Created ad group with resource_name: {message.resource_name}.")
        # [END handle_partial_failure_2]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Adds an ad group for specified customer and campaign id."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-i", "--campaign_id", type=str, required=True, help="The campaign ID."
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    main(googleads_client, args.customer_id, args.campaign_id)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "AppBiddingGoalEnum",
    },
)


class AppBiddingGoalEnum(proto.Message):
    r"""Container for enum describing an app bidding goal for raise
    Target CPA recommendation.

    """

    class AppBiddingGoal(proto.Enum):
        r"""Represents the goal towards which the bidding strategy, of an
        app campaign, should optimize for.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Represents value unknown in this version of
                the API.
            OPTIMIZE_FOR_INSTALL_CONVERSION_VOLUME (2):
                The bidding strategy of the app campaign
                should aim to maximize installation of the app.
            OPTIMIZE_FOR_IN_APP_CONVERSION_VOLUME (3):
                The bidding strategy of the app campaign
                should aim to maximize the selected in-app
                conversions' volume.
            OPTIMIZE_FOR_TOTAL_CONVERSION_VALUE (4):
                The bidding strategy of the app campaign
                should aim to maximize all conversions' value,
                that is, install and selected in-app
                conversions.
            OPTIMIZE_FOR_TARGET_IN_APP_CONVERSION (5):
                The bidding strategy of the app campaign
                should aim to maximize just the selected in-app
                conversion's volume, while achieving or
                exceeding target cost per in-app conversion.
            OPTIMIZE_FOR_RETURN_ON_ADVERTISING_SPEND (6):
                The bidding strategy of the app campaign
                should aim to maximize all conversions' value,
                that is, install and selected in-app conversions
                while achieving or exceeding target return on
                advertising spend.
            OPTIMIZE_FOR_INSTALL_CONVERSION_VOLUME_WITHOUT_TARGET_CPI (7):
                This bidding strategy of the app campaign
                should aim to maximize installation of the app
                without advertiser-provided target
                cost-per-install.
            OPTIMIZE_FOR_PRE_REGISTRATION_CONVERSION_VOLUME (8):
                This bidding strategy of the app campaign
                should aim to maximize pre-registration of the
                app.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OPTIMIZE_FOR_INSTALL_CONVERSION_VOLUME = 2
        OPTIMIZE_FOR_IN_APP_CONVERSION_VOLUME = 3
        OPTIMIZE_FOR_TOTAL_CONVERSION_VALUE = 4
        OPTIMIZE_FOR_TARGET_IN_APP_CONVERSION = 5
        OPTIMIZE_FOR_RETURN_ON_ADVERTISING_SPEND = 6
        OPTIMIZE_FOR_INSTALL_CONVERSION_VOLUME_WITHOUT_TARGET_CPI = 7
        OPTIMIZE_FOR_PRE_REGISTRATION_CONVERSION_VOLUME = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "ExtensionSettingErrorEnum",
    },
)


class ExtensionSettingErrorEnum(proto.Message):
    r"""Container for enum describing validation errors of extension
    settings.

    """

    class ExtensionSettingError(proto.Enum):
        r"""Enum describing possible extension setting errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            EXTENSIONS_REQUIRED (2):
                A platform restriction was provided without
                input extensions or existing extensions.
            FEED_TYPE_EXTENSION_TYPE_MISMATCH (3):
                The provided feed type does not correspond to
                the provided extensions.
            INVALID_FEED_TYPE (4):
                The provided feed type cannot be used.
            INVALID_FEED_TYPE_FOR_CUSTOMER_EXTENSION_SETTING (5):
                The provided feed type cannot be used at the
                customer level.
            CANNOT_CHANGE_FEED_ITEM_ON_CREATE (6):
                Cannot change a feed item field on a CREATE
                operation.
            CANNOT_UPDATE_NEWLY_CREATED_EXTENSION (7):
                Cannot update an extension that is not
                already in this setting.
            NO_EXISTING_AD_GROUP_EXTENSION_SETTING_FOR_TYPE (8):
                There is no existing AdGroupExtensionSetting
                for this type.
            NO_EXISTING_CAMPAIGN_EXTENSION_SETTING_FOR_TYPE (9):
                There is no existing CampaignExtensionSetting
                for this type.
            NO_EXISTING_CUSTOMER_EXTENSION_SETTING_FOR_TYPE (10):
                There is no existing CustomerExtensionSetting
                for this type.
            AD_GROUP_EXTENSION_SETTING_ALREADY_EXISTS (11):
                The AdGroupExtensionSetting already exists.
                UPDATE should be used to modify the existing
                AdGroupExtensionSetting.
            CAMPAIGN_EXTENSION_SETTING_ALREADY_EXISTS (12):
                The CampaignExtensionSetting already exists.
                UPDATE should be used to modify the existing
                CampaignExtensionSetting.
            CUSTOMER_EXTENSION_SETTING_ALREADY_EXISTS (13):
                The CustomerExtensionSetting already exists.
                UPDATE should be used to modify the existing
                CustomerExtensionSetting.
            AD_GROUP_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE (14):
                An active ad group feed already exists for
                this place holder type.
            CAMPAIGN_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE (15):
                An active campaign feed already exists for
                this place holder type.
            CUSTOMER_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE (16):
                An active customer feed already exists for
                this place holder type.
            VALUE_OUT_OF_RANGE (17):
                Value is not within the accepted range.
            CANNOT_SET_FIELD_WITH_FINAL_URLS (18):
                Cannot simultaneously set specified field
                with final urls.
            FINAL_URLS_NOT_SET (19):
                Must set field with final urls.
            INVALID_PHONE_NUMBER (20):
                Phone number for a call extension is invalid.
            PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY (21):
                Phone number for a call extension is not
                supported for the given country code.
            CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED (22):
                A carrier specific number in short format is
                not allowed for call extensions.
            PREMIUM_RATE_NUMBER_NOT_ALLOWED (23):
                Premium rate numbers are not allowed for call
                extensions.
            DISALLOWED_NUMBER_TYPE (24):
                Phone number type for a call extension is not
                allowed.
            INVALID_DOMESTIC_PHONE_NUMBER_FORMAT (25):
                Phone number for a call extension does not
                meet domestic format requirements.
            VANITY_PHONE_NUMBER_NOT_ALLOWED (26):
                Vanity phone numbers (for example, those
                including letters) are not allowed for call
                extensions.
            INVALID_COUNTRY_CODE (27):
                Country code provided for a call extension is
                invalid.
            INVALID_CALL_CONVERSION_TYPE_ID (28):
                Call conversion type id provided for a call
                extension is invalid.
            CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING (69):
                For a call extension, the customer is not on
                the allow-list for call tracking.
            CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY (30):
                Call tracking is not supported for the given
                country for a call extension.
            INVALID_APP_ID (31):
                App id provided for an app extension is
                invalid.
            QUOTES_IN_REVIEW_EXTENSION_SNIPPET (32):
                Quotation marks present in the review text
                for a review extension.
            HYPHENS_IN_REVIEW_EXTENSION_SNIPPET (33):
                Hyphen character present in the review text
                for a review extension.
            REVIEW_EXTENSION_SOURCE_NOT_ELIGIBLE (34):
                A blocked review source name or url was
                provided for a review extension.
            SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT (35):
                Review source name should not be found in the
                review text.
            MISSING_FIELD (36):
                Field must be set.
            INCONSISTENT_CURRENCY_CODES (37):
                Inconsistent currency codes.
            PRICE_EXTENSION_HAS_DUPLICATED_HEADERS (38):
                Price extension cannot have duplicated
                headers.
            PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION (39):
                Price item cannot have duplicated header and
                description.
            PRICE_EXTENSION_HAS_TOO_FEW_ITEMS (40):
                Price extension has too few items
            PRICE_EXTENSION_HAS_TOO_MANY_ITEMS (41):
                Price extension has too many items
            UNSUPPORTED_VALUE (42):
                The input value is not currently supported.
            INVALID_DEVICE_PREFERENCE (43):
                Unknown or unsupported device preference.
            INVALID_SCHEDULE_END (45):
                Invalid feed item schedule end time (for
                example, endHour = 24 and endMinute != 0).
            DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE (47):
                Date time zone does not match the account's
                time zone.
            OVERLAPPING_SCHEDULES_NOT_ALLOWED (48):
                Overlapping feed item schedule times (for
                example, 7-10AM and 8-11AM) are not allowed.
            SCHEDULE_END_NOT_AFTER_START (49):
                Feed item schedule end time must be after
                start time.
            TOO_MANY_SCHEDULES_PER_DAY (50):
                There are too many feed item schedules per
                day.
            DUPLICATE_EXTENSION_FEED_ITEM_EDIT (51):
                Cannot edit the same extension feed item more
                than once in the same request.
            INVALID_SNIPPETS_HEADER (52):
                Invalid structured snippet header.
            PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY (53):
                Phone number with call tracking enabled is
                not supported for the specified country.
            CAMPAIGN_TARGETING_MISMATCH (54):
                The targeted adgroup must belong to the
                targeted campaign.
            CANNOT_OPERATE_ON_REMOVED_FEED (55):
                The feed used by the ExtensionSetting is
                removed and cannot be operated on. Remove the
                ExtensionSetting to allow a new one to be
                created using an active feed.
            EXTENSION_TYPE_REQUIRED (56):
                The ExtensionFeedItem type is required for
                this operation.
            INCOMPATIBLE_UNDERLYING_MATCHING_FUNCTION (57):
                The matching function that links the
                extension feed to the customer, campaign, or ad
                group is not compatible with the
                ExtensionSetting services.
            START_DATE_AFTER_END_DATE (58):
                Start date must be before end date.
            INVALID_PRICE_FORMAT (59):
                Input price is not in a valid format.
            PROMOTION_INVALID_TIME (60):
                The promotion time is invalid.
            PROMOTION_CANNOT_SET_PERCENT_DISCOUNT_AND_MONEY_DISCOUNT (61):
                Cannot set both percent discount and money
                discount fields.
            PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT (62):
                Cannot set both promotion code and orders
                over amount fields.
            TOO_MANY_DECIMAL_PLACES_SPECIFIED (63):
                This field has too many decimal places
                specified.
            INVALID_LANGUAGE_CODE (64):
                The language code is not valid.
            UNSUPPORTED_LANGUAGE (65):
                The language is not supported.
            CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED (66):
                Customer hasn't consented for call recording,
                which is required for adding/updating call
                extensions. See
                https://support.google.com/google-ads/answer/7412639.
            EXTENSION_SETTING_UPDATE_IS_A_NOOP (67):
                The UPDATE operation does not specify any
                fields other than the resource name in the
                update mask.
            DISALLOWED_TEXT (68):
                The extension contains text which has been
                prohibited on policy grounds.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EXTENSIONS_REQUIRED = 2
        FEED_TYPE_EXTENSION_TYPE_MISMATCH = 3
        INVALID_FEED_TYPE = 4
        INVALID_FEED_TYPE_FOR_CUSTOMER_EXTENSION_SETTING = 5
        CANNOT_CHANGE_FEED_ITEM_ON_CREATE = 6
        CANNOT_UPDATE_NEWLY_CREATED_EXTENSION = 7
        NO_EXISTING_AD_GROUP_EXTENSION_SETTING_FOR_TYPE = 8
        NO_EXISTING_CAMPAIGN_EXTENSION_SETTING_FOR_TYPE = 9
        NO_EXISTING_CUSTOMER_EXTENSION_SETTING_FOR_TYPE = 10
        AD_GROUP_EXTENSION_SETTING_ALREADY_EXISTS = 11
        CAMPAIGN_EXTENSION_SETTING_ALREADY_EXISTS = 12
        CUSTOMER_EXTENSION_SETTING_ALREADY_EXISTS = 13
        AD_GROUP_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE = 14
        CAMPAIGN_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE = 15
        CUSTOMER_FEED_ALREADY_EXISTS_FOR_PLACEHOLDER_TYPE = 16
        VALUE_OUT_OF_RANGE = 17
        CANNOT_SET_FIELD_WITH_FINAL_URLS = 18
        FINAL_URLS_NOT_SET = 19
        INVALID_PHONE_NUMBER = 20
        PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY = 21
        CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED = 22
        PREMIUM_RATE_NUMBER_NOT_ALLOWED = 23
        DISALLOWED_NUMBER_TYPE = 24
        INVALID_DOMESTIC_PHONE_NUMBER_FORMAT = 25
        VANITY_PHONE_NUMBER_NOT_ALLOWED = 26
        INVALID_COUNTRY_CODE = 27
        INVALID_CALL_CONVERSION_TYPE_ID = 28
        CUSTOMER_NOT_IN_ALLOWLIST_FOR_CALLTRACKING = 69
        CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY = 30
        INVALID_APP_ID = 31
        QUOTES_IN_REVIEW_EXTENSION_SNIPPET = 32
        HYPHENS_IN_REVIEW_EXTENSION_SNIPPET = 33
        REVIEW_EXTENSION_SOURCE_NOT_ELIGIBLE = 34
        SOURCE_NAME_IN_REVIEW_EXTENSION_TEXT = 35
        MISSING_FIELD = 36
        INCONSISTENT_CURRENCY_CODES = 37
        PRICE_EXTENSION_HAS_DUPLICATED_HEADERS = 38
        PRICE_ITEM_HAS_DUPLICATED_HEADER_AND_DESCRIPTION = 39
        PRICE_EXTENSION_HAS_TOO_FEW_ITEMS = 40
        PRICE_EXTENSION_HAS_TOO_MANY_ITEMS = 41
        UNSUPPORTED_VALUE = 42
        INVALID_DEVICE_PREFERENCE = 43
        INVALID_SCHEDULE_END = 45
        DATE_TIME_MUST_BE_IN_ACCOUNT_TIME_ZONE = 47
        OVERLAPPING_SCHEDULES_NOT_ALLOWED = 48
        SCHEDULE_END_NOT_AFTER_START = 49
        TOO_MANY_SCHEDULES_PER_DAY = 50
        DUPLICATE_EXTENSION_FEED_ITEM_EDIT = 51
        INVALID_SNIPPETS_HEADER = 52
        PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY = 53
        CAMPAIGN_TARGETING_MISMATCH = 54
        CANNOT_OPERATE_ON_REMOVED_FEED = 55
        EXTENSION_TYPE_REQUIRED = 56
        INCOMPATIBLE_UNDERLYING_MATCHING_FUNCTION = 57
        START_DATE_AFTER_END_DATE = 58
        INVALID_PRICE_FORMAT = 59
        PROMOTION_INVALID_TIME = 60
        PROMOTION_CANNOT_SET_PERCENT_DISCOUNT_AND_MONEY_DISCOUNT = 61
        PROMOTION_CANNOT_SET_PROMOTION_CODE_AND_ORDERS_OVER_AMOUNT = 62
        TOO_MANY_DECIMAL_PLACES_SPECIFIED = 63
        INVALID_LANGUAGE_CODE = 64
        UNSUPPORTED_LANGUAGE = 65
        CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED = 66
        EXTENSION_SETTING_UPDATE_IS_A_NOOP = 67
        DISALLOWED_TEXT = 68


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AssetLinkPrimaryStatusReasonEnum",
    },
)


class AssetLinkPrimaryStatusReasonEnum(proto.Message):
    r"""Provides the reason of a primary status.
    For example: a sitelink may be paused for a particular campaign.

    """

    class AssetLinkPrimaryStatusReason(proto.Enum):
        r"""Enum Provides insight into why an asset is not serving or not
        serving at full capacity for a particular link level. These
        reasons are aggregated to determine a final PrimaryStatus.
        For example, a sitelink might be paused by the user, but also
        limited in serving due to violation of an alcohol policy. In
        this case, the PrimaryStatus will be returned as PAUSED, since
        the asset's effective status is determined by its paused state.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ASSET_LINK_PAUSED (2):
                The asset is paused for its linked rollup
                level. Contributes to a PrimaryStatus of PAUSED.
            ASSET_LINK_REMOVED (3):
                The asset is removed for its linked rollup
                level. Contributes to a PrimaryStatus of
                REMOVED.
            ASSET_DISAPPROVED (4):
                The asset has been marked as disapproved. Contributes to a
                PrimaryStatus of NOT_ELIGIBLE
            ASSET_UNDER_REVIEW (5):
                The asset has not completed policy review.
                Contributes to a PrimaryStatus of PENDING.
            ASSET_APPROVED_LABELED (6):
                The asset is approved with policies applied.
                Contributes to a PrimaryStatus of LIMITED.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ASSET_LINK_PAUSED = 2
        ASSET_LINK_REMOVED = 3
        ASSET_DISAPPROVED = 4
        ASSET_UNDER_REVIEW = 5
        ASSET_APPROVED_LABELED = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import linked_product_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "ProductLink",
        "DataPartnerIdentifier",
        "GoogleAdsIdentifier",
        "MerchantCenterIdentifier",
        "AdvertisingPartnerIdentifier",
    },
)


class ProductLink(proto.Message):
    r"""Represents the data sharing connection between  a Google
    Ads customer and another product.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. Resource name of the product link. ProductLink
            resource names have the form:

            ``customers/{customer_id}/productLinks/{product_link_id}``
        product_link_id (int):
            Output only. The ID of the link.
            This field is read only.

            This field is a member of `oneof`_ ``_product_link_id``.
        type_ (google.ads.googleads.v21.enums.types.LinkedProductTypeEnum.LinkedProductType):
            Output only. The type of the linked product.
        data_partner (google.ads.googleads.v21.resources.types.DataPartnerIdentifier):
            Immutable. Data partner link.

            This field is a member of `oneof`_ ``linked_product``.
        google_ads (google.ads.googleads.v21.resources.types.GoogleAdsIdentifier):
            Immutable. Google Ads link.

            This field is a member of `oneof`_ ``linked_product``.
        merchant_center (google.ads.googleads.v21.resources.types.MerchantCenterIdentifier):
            Immutable. Google Merchant Center link.

            This field is a member of `oneof`_ ``linked_product``.
        advertising_partner (google.ads.googleads.v21.resources.types.AdvertisingPartnerIdentifier):
            Output only. Advertising Partner link.

            This field is a member of `oneof`_ ``linked_product``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    product_link_id: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    type_: linked_product_type.LinkedProductTypeEnum.LinkedProductType = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=linked_product_type.LinkedProductTypeEnum.LinkedProductType,
        )
    )
    data_partner: "DataPartnerIdentifier" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="linked_product",
        message="DataPartnerIdentifier",
    )
    google_ads: "GoogleAdsIdentifier" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="linked_product",
        message="GoogleAdsIdentifier",
    )
    merchant_center: "MerchantCenterIdentifier" = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="linked_product",
        message="MerchantCenterIdentifier",
    )
    advertising_partner: "AdvertisingPartnerIdentifier" = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="linked_product",
        message="AdvertisingPartnerIdentifier",
    )


class DataPartnerIdentifier(proto.Message):
    r"""The identifier for Data Partner account.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        data_partner_id (int):
            Immutable. The customer ID of the Data
            partner account. This field is required and
            should not be empty when creating a new data
            partner link. It is unable to be modified after
            the creation of the link.

            This field is a member of `oneof`_ ``_data_partner_id``.
    """

    data_partner_id: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )


class GoogleAdsIdentifier(proto.Message):
    r"""The identifier for Google Ads account.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer (str):
            Immutable. The resource name of the Google
            Ads account. This field is required and should
            not be empty when creating a new Google Ads
            link. It is unable to be modified after the
            creation of the link.

            This field is a member of `oneof`_ ``_customer``.
    """

    customer: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class MerchantCenterIdentifier(proto.Message):
    r"""The identifier for Google Merchant Center account

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        merchant_center_id (int):
            Immutable. The customer ID of the Google
            Merchant Center account. This field is required
            and should not be empty when creating a new
            Merchant Center link. It is unable to be
            modified after the creation of the link.

            This field is a member of `oneof`_ ``_merchant_center_id``.
    """

    merchant_center_id: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )


class AdvertisingPartnerIdentifier(proto.Message):
    r"""The identifier for the Advertising Partner Google Ads
    account.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer (str):
            Output only. The resource name of the
            advertising partner Google Ads account. This
            field is required and should not be empty when
            creating a new Advertising Partner link. It is
            unable to be modified after the creation of the
            link.

            This field is a member of `oneof`_ ``_customer``.
    """

    customer: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
